# Description: Shows how to use graphs
# Category:    general
# Classes:     Graph
# Uses:
# Referenced:  graph.htm

import Orange.network

graph = Orange.network.GraphAsMatrix(4, 0)
graph.objects = ["Age", "Gender", "Height", "Weight"]

graph["Age", "Gender"] = 0.1
graph["Age", "Height"] = 1.2
graph["Gender", "Height"] = 0.3
print(graph.getEdges())

print(graph[1, 2])
print(graph["Gender", "Height"])

graph.objects = {}
graph.objects["Age"] = 0
graph.objects["Gender"] = 1
graph.objects["Height"] = 2

print(graph["Age", "Gender"])
try:
    print(graph["Gender", "Height"])
except:
    print('graph["Gender", "Height"] failed')

print(graph.getNeighbours("Age"))
graph.returnIndices = 1
print(graph.getNeighbours("Age"))


graph = Orange.network.GraphAsMatrix(5, 0, 3)
print(graph[4, 1])
graph[4, 1, 1]=12
print(graph[4, 1, 1])
print(graph[4, 1])

print(graph.edgeExists(4, 1))
print(graph.edgeExists(4, 2))
print(graph.edgeExists(4, 1, 1))
print(graph.edgeExists(4, 1, 2))

e = graph[4, 1]
e[1]
e[2]
e[1] = None
e[2] = 3
print(graph.edgeExists(4, 1, 1))
print(graph.edgeExists(4, 1, 2))
print(e)
graph[4, 1]=None
print(e)




import numpy as np

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import QLineF, QPointF, QRectF, Qt
from PyQt4.QtGui import qApp, QBrush, QPen, QPolygonF, QStyle, QColor

from orangecontrib.network._fr_layout import fruchterman_reingold_layout

import pyqtgraph as pg

# Expose OpenGL rendering for large graphs, if available
HAVE_OPENGL = True
try: from PyQt4 import QtOpenGL
except: HAVE_OPENGL = False


class QGraphicsEdge(QtGui.QGraphicsLineItem):
    def __init__(self, source, dest, view=None):
        super().__init__()
        self.setAcceptedMouseButtons(Qt.NoButton)
        self.setFlags(self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity)
        self.setZValue(1)
        self.setPen(QPen(Qt.darkGray, .5))

        source.addEdge(self)
        dest.addEdge(self)
        self.source = source
        self.dest = dest
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('test', self)
        label.setVisible(False)
        label.setBrush(Qt.gray)
        label.setZValue(2)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        view.scene().addItem(label)

        self.adjust()

    def adjust(self):
        line = QLineF(self.mapFromItem(self.source, 0, 0),
                      self.mapFromItem(self.dest, 0, 0))
        self.label.setPos(line.pointAt(.5))
        self.setLine(self.__transform().map(line))


class Edge(QGraphicsEdge):
    def __init__(self, source, dest, view=None):
        super().__init__(source, dest, view)
        self.setSize(.5)

    def setSize(self, size):
        self.setPen(QPen(self.pen().color(), size))

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color):
        self.setPen(QPen(QColor(color), self.pen().width()))


class QGraphicsNode(QtGui.QGraphicsEllipseItem):
    """This class is the bare minimum to sustain a connected graph"""
    def __init__(self, rect=QRectF(-5, -5, 10, 10), view=None):
        super().__init__(rect)
        self.setCacheMode(self.DeviceCoordinateCache)
        self.setAcceptHoverEvents(True)
        self.setFlags(self.ItemIsMovable |
                      self.ItemIsSelectable |
                      self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity |
                      self.ItemSendsGeometryChanges)
        self.setZValue(4)

        self.edges = []
        self._radius = rect.width() / 2
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('test', self)
        #~ label.setVisible(False)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        label.setZValue(3)
        view.scene().addItem(label)

    def setPos(self, x, y):
        self.adjust()
        super().setPos(x, y)

    def adjust(self):
        # Adjust label position
        d = 1 / self.__transform().m11() * self._radius
        self.label.setPos(self.pos().x() + d, self.pos().y() + d)

    def addEdge(self, edge):
        self.edges.append(edge)

    def itemChange(self, change, value):
        if change == self.ItemPositionHasChanged:
            self.adjust()
            for edge in self.edges:
                edge.adjust()
        return super().itemChange(change, value)


class Node(QGraphicsNode):
    """
    This class provides an interface for all the bells & whistles of the
    Network Explorer.
    """

    BRUSH_DEFAULT = QBrush(QColor('#669'))

    class Pen:
        DEFAULT = QPen(Qt.black, 0)
        SELECTED = QPen(QColor('#ee0000'), 3)
        HIGHLIGHTED = QPen(QColor('#ff7700'), 3)

    _tooltip = lambda _: ''

    def __init__(self, id, view):
        super().__init__(view=view)
        self.id = id
        self.setBrush(Node.BRUSH_DEFAULT)
        self.setPen(Node.Pen.DEFAULT)

        self._is_highlighted = False

    def setSize(self, size):
        self._radius = radius = size/2
        self.setRect(-radius, -radius, size, size)

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color=None):
        self.setBrush(QBrush(QColor(color)) if color else Node.BRUSH_DEFAULT)

    def isHighlighted(self):
        return self._is_highlighted
    def setHighlighted(self, highlight):
        self._is_highlighted = highlight
        if not self.isSelected():
            self.itemChange(self.ItemSelectedChange, False)

    def itemChange(self, change, value):
        if change == self.ItemSelectedChange:
            self.setPen(Node.Pen.SELECTED if value else
                        Node.Pen.HIGHLIGHTED if self._is_highlighted else
                        Node.Pen.DEFAULT)
        return super().itemChange(change, value)

    def paint(self, painter, option, widget):
        option.state &= ~QStyle.State_Selected  # We use a custom selection pen
        super().paint(painter, option, widget)

    def setTooltip(self, callback):
        self._tooltip = callback
    def hoverEnterEvent(self, event):
        self.setToolTip(self._tooltip())
    def hoverLeaveEvent(self, event):
        self.setToolTip('');


class TextWidget(QtGui.QGraphicsWidget):
    def __init__(self, view):
        super().__init__(view)
        #~ self.setCacheMode(self.DeviceCoordinateCache)
        #~ self.setFlags(self.ItemIgnoresTransformations |
                      #~ self.ItemIgnoresParentOpacity)
        #~ self.setZValue(5)
        #~ self.setAcceptHoverEvents(False)

        self.view = view
        self.text = QtGui.QGraphicsTextItem()

    def setText(self, html):
        self.text.setHtml(html)
        #~ super().setText(html)

    def adjust(self):
        #~ d = 1 / self.__transform().m11() * self._radius
        #~ self.label.setPos(self.pos().x() + d, self.pos().y() + d)
        self.setPos(self.view.mapToScene(10, 8))

class GraphView(QtGui.QGraphicsView):

    positionsChanged = QtCore.pyqtSignal(np.ndarray)
    # Emitted when nodes' selected or highlighted state changes
    selectionChanged = QtCore.pyqtSignal()

    def __init__(self):
        super().__init__()
        self.nodes = []
        self.edges = []
        self._selection = []
        self._clicked_node = None

        scene = QtGui.QGraphicsScene(self)
        scene.setItemIndexMethod(scene.NoIndex)
        #~ scene.setSceneRect(-400, -400, 800, 800)
        #~ scene.setSceneRect(-1, -1, 2, 2)
        self.setScene(scene)
        rect = QRectF(QtGui.QDesktopWidget().geometry())
        #~ self.scene().setSceneRect(rect.x(), rect.y(), 2*rect.width(), 2*rect.height())
        #~ self.setSceneRect(rect.x(), rect.y(), 2*rect.width(), 2*rect.height())

        #~ self.setSceneRect(-50, -50, 100, 100)
        #~ r = self.scene().sceneRect()
        #~ print(r.x())
        #~ print(r.y())
        #~ print(r.width())
        #~ print(r.height())
        self.setCacheMode(self.CacheBackground)
        self.setViewportUpdateMode(self.BoundingRectViewportUpdate)

        self.setTransformationAnchor(self.AnchorUnderMouse)
        self.setResizeAnchor(self.AnchorViewCenter)
        #~ self.setResizeAnchor(self.AnchorUnderMouse)

        #~ self.setDragMode(self.ScrollHandDrag)
        self.setDragMode(self.RubberBandDrag)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        #~ self.setMinimumSize(400, 400)
        #~ self.positionsChanged.connect(self._update_positions, type=Qt.BlockingQueuedConnection)
        self.positionsChanged.connect(self._update_positions)

        self.text = None

    def mousePressEvent(self, event):
        event.ignore()
        if event.button() == Qt.LeftButton:
            self.setDragMode(self.NoDrag)
            self.setDragMode(self.RubberBandDrag)
            if self._is_animating:
                self._is_animating = False
                self.setCursor(Qt.ArrowCursor)
            # Save the current selection and restore it on mouse{Move,Release}
            self._selection = []
            node = self._clicked_node = self.itemAt(event.pos())
            if event.modifiers() & (Qt.ControlModifier | Qt.ShiftModifier):
                self._selection = self.scene().selectedItems()
        # On right mouse button, switch to pan mode
        elif event.button() == Qt.RightButton:
            self.setDragMode(self.ScrollHandDrag)
            # Forge left mouse button event
            event = QtGui.QMouseEvent(event.type(),
                                      event.pos(),
                                      Qt.LeftButton,
                                      event.buttons() | Qt.LeftButton,
                                      event.modifiers())
        super().mousePressEvent(event)
        # Reselect the selection that had just been discarded
        for node in self._selection: node.setSelected(True)
    def mouseMoveEvent(self, event):
        super().mouseMoveEvent(event)
        if not self._clicked_node:
            for node in self._selection: node.setSelected(True)
    def mouseReleaseEvent(self, event):
        super().mouseReleaseEvent(event)
        if self.dragMode() == self.RubberBandDrag:
            for node in self._selection: node.setSelected(True)
            self.selectionChanged.emit()
        self.setDragMode(self.NoDrag)
        self.setDragMode(self.RubberBandDrag)



    def getSelected(self):
        return self.scene().selectedItems()
    def getHighlighted(self):
        return [node for node in self.nodes
                if node.isHighlighted() and not node.isSelected()]
    def setHighlighted(self, nodes):
        for node in nodes:
            node.setHighlighted(True)
        self.selectionChanged.emit()

    def clear(self):
        self.scene().clear()
        self.scene().setSceneRect(QRectF())
        self.nodes.clear()
        self.edges.clear()

    def set_graph(self, graph):
        assert not graph or isinstance(graph, nx.Graph)  # TODO
        self.graph = graph
        if not graph:
            self.clear()
            return
        large_graph = graph.number_of_nodes() + graph.number_of_edges() > 4000
        self.setViewport(QtOpenGL.QGLWidget()
                         if large_graph and HAVE_OPENGL else
                         QtGui.QWidget())
        if not large_graph:
            self.setRenderHints(QtGui.QPainter.Antialiasing |
                                QtGui.QPainter.TextAntialiasing)
        self.clear()
        for v in sorted(graph.nodes()):
            self.addNode(Node(v, view=self))
        for u, v in graph.edges():
            self.addEdge(Edge(self.nodes[u], self.nodes[v], view=self))
        #~ self.ensureVisible(self.scene().sceneRect())
        #~ self.centerOn(self.scene().itemsBoundingRect().center())
        self.setText('<b>Yeah</b>, this works!')
        self.centerOn(self.scene().sceneRect().center())
        self.selectionChanged.emit()
        self.relayout()

    def addNode(self, node):
        assert isinstance(node, Node)
        self.nodes.append(node)
        self.scene().addItem(node)
    def addEdge(self, edge):
        assert isinstance(edge, Edge)
        self.edges.append(edge)
        self.scene().addItem(edge)

    def setText(self, text):
        self.text = QtGui.QStaticText(text)
    def drawForeground(self, painter, rect):
        if self.text:
            painter.resetTransform()
            painter.drawStaticText(10, 18, self.text)
        super().drawForeground(painter, rect)
    def scrollContentsBy(self, dx, dy):
        scene = self.scene()
        scene.invalidate(layers=scene.BackgroundLayer)
        super().scrollContentsBy(dx, dy)

    def wheelEvent(self, event):
        if event.orientation() != Qt.Vertical: return
        self.scaleView(2**(event.delta() / 240))
    def scaleView(self, factor):
        magnitude = self.matrix().scale(factor, factor).mapRect(QRectF(0, 0, 1, 1)).width()
        if 0.2 < magnitude < 30:
            self.scale(factor, factor)
        # Reposition nodes' labela and edges, both of which are node-dependend
        # (and nodes just "moved")
        for node in self.nodes: node.adjust()
        for edge in self.edges: edge.adjust()
        #~ self.text.adjust()

    def relayout(self, randomize=True):
        self._is_animating = True
        self.setCursor(Qt.ForbiddenCursor)
        pos = None
        if not randomize:
            pos = [[pos.x(), pos.y()]
                   for pos in (node.pos()
                               for node in self.nodes)]
        fruchterman_reingold_layout(self.graph,
                                    pos=pos,
                                    iterations=50,
                                    callback=self.update_positions)
        self._is_animating = False
        self.setCursor(Qt.ArrowCursor)

    def update_positions(self, positions):
        self.positionsChanged.emit(positions)
        return self._is_animating
    def _update_positions(self, positions):
        positions = positions*300
        MARGIN = 500
        (x, y), (w, h) = np.min(positions, 0) - MARGIN, np.ptp(positions, 0) + 2*MARGIN
        self.scene().setSceneRect(x, y, w, h)
        for node, pos in zip(self.nodes, positions):
            node.setPos(*pos)
        qApp.processEvents()

    def drawBackground(self, painter, rect):
        # TODO: delete
        painter.setPen(QPen(Qt.red, 2))
        painter.setBrush(QBrush('yellow'))
        #~ print(self.scene().sceneRect())
        painter.drawRect(self.scene().sceneRect())

class GVTest(QtGui.QGraphicsView):
    def __init__(self):
        super().__init__()
        scene = QtGui.QGraphicsScene(self)
        scene.setSceneRect(0, 0, 1000, 1000)
        self.setScene(scene)

    def mouseReleaseEvent(self, event):
        print(event)
        self.setDragMode(self.RubberBandDrag)
        super().mouseReleaseEvent(event)

    def test(self):
        QMouseEvent, QEvent = QtGui.QMouseEvent, QtCore.QEvent
        from PyQt4.QtCore import QPoint

        item = self.scene().addRect(200, 200, 100, 100)
        item.setFlag(item.ItemIsMovable)
        item.setFlag(item.ItemIsSelectable)
        item.setSelected(True)

        rect = item.boundingRect()
        print(rect.x(), rect.y())

        self.setDragMode(self.ScrollHandDrag)
        self.centerOn(item.boundingRect().center())
        click = QMouseEvent(QEvent.MouseButtonPress,
                            QPoint(250, 250),
                            Qt.LeftButton,
                            Qt.LeftButton,
                            Qt.NoModifier)
        drag = QMouseEvent(QEvent.MouseMove,
                            QPoint(50, 50),
                            Qt.LeftButton,
                            Qt.LeftButton,
                            Qt.NoModifier)
        release = QMouseEvent(QEvent.MouseButtonRelease,
                              QPoint(50, 50),
                            Qt.LeftButton,
                            Qt.LeftButton,
                            Qt.NoModifier)
        qApp.sendEvent(self.viewport(), click)
        qApp.sendEvent(self.viewport(), drag)
        qApp.sendEvent(self.viewport(), release)

        print(item.pos().x(), item.pos().y())

        #~ self.setDragMode(self.RubberBandDrag)


if __name__ == '__main__':

    import sys
    app = QtGui.QApplication(sys.argv)

    view = GVTest()
    view.show()
    view.test()
    sys.exit(app.exec_())



    widget = GraphView()
    widget.show()
    import networkx as nx
    G = nx.scale_free_graph(int(sys.argv[1]) if len(sys.argv) > 1 else 100, seed=0)
    #~ G = nx.cycle_graph(int(sys.argv[1]) if len(sys.argv) > 1 else 100)

    #~ widget.relayout()
    widget.set_graph(G)
    print('nodes', len(widget.nodes), 'edges', len(widget.edges))
    #~ from threading import Thread
    #~ Thread(target=fruchterman_reingold_layout, args=(G,), kwargs=dict(iterations=50, callback=widget.update_positions)).start()

    for node in widget.nodes[:10]:
        node.setHighlighted(True)

    sys.exit(app.exec_())



import os
from operator import itemgetter, add
from functools import reduce, wraps
from itertools import chain
from threading import Lock

import numpy as np

from PyQt4.QtGui import *
from PyQt4.QtCore import *

import Orange
from Orange.util import scale
from Orange.widgets import gui, widget, settings
from Orange.widgets.utils.colorpalette import ColorPaletteDlg
from Orange.widgets.unsupervised.owmds import torgerson as MDS
from Orange.widgets.utils.colorpalette import ColorPaletteGenerator, GradientPaletteGenerator

from Orange.data import Table, Domain, DiscreteVariable, StringVariable
import orangecontrib.network as network
from orangecontrib.network.widgets.graphview import GraphView, Node, FR_ITERATIONS


def non_reentrant(func):
    """Prevent the function from reentry."""
    lock = Lock()
    @wraps(func)
    def locker(*args, **kwargs):
        if lock.acquire(False):
            try: return func(*args, **kwargs)
            finally: lock.release()
    return locker


CONTINUOUS_PALETTE = GradientPaletteGenerator('#00ffff', '#550066')


class SelectionMode:
    NONE,       \
    SEARCH,     \
    NEIGHBORS,  \
    AT_LEAST_N, \
    AT_MOST_N,  \
    ANY_NEIGH,  \
    AVG_NEIGH,  \
    MOST_CONN,  \
    FROM_INPUT  \
    = range(9)  # FML


class Output:
    SUBGRAPH = 'Selected sub-network'
    DISTANCE = 'Distance matrix'
    SELECTED = 'Selected items'
    HIGHLIGHTED = 'Highlighted items'
    REMAINING = 'Remaining items'
    all = (SUBGRAPH, DISTANCE, SELECTED, HIGHLIGHTED, REMAINING)


class OWNxExplorer(widget.OWWidget):
    name = "Network Explorer"
    description = "Visually explore the network and its properties."
    icon = "icons/NetworkExplorer.svg"
    priority = 6420

    inputs = [
        ("Network", network.Graph, "set_graph", widget.Default),
        ("Node Subset", Table, 'set_marking_items'),
        ("Node Data", Table, "set_items"),
        ("Node Distances", Orange.misc.DistMatrix, "set_items_distance_matrix"),
    ]

    outputs = [(Output.SUBGRAPH, network.Graph),
               (Output.DISTANCE, Orange.misc.DistMatrix),
               (Output.SELECTED, Table),
               (Output.HIGHLIGHTED, Table),
               (Output.REMAINING, Table)]

    settingsList = ["lastVertexSizeColumn", "lastColorColumn",
                    "lastLabelColumns", "lastTooltipColumns",]
    # TODO: set settings

    UserAdviceMessages = [
        widget.Message('When selecting nodes on the Marking tab, '
                       'press <b><tt>Enter</tt></b> key to add '
                       '<b><font color="{}">highlighted</font></b> nodes to '
                       '<b><font color="{}">selection</font></b>.'
                       .format(Node.Pen.HIGHLIGHTED.color().name(),
                               Node.Pen.SELECTED.color().name()),
                       'marking-info',
                       widget.Message.Information),
        widget.Message('Left-click to select nodes '
                       '(hold <b><tt>Shift</tt></b> to append to selection). '
                       'Right-click to pan/move the view. Scroll to zoom.',
                       'mouse-info',
                       widget.Message.Information),
    ]

    do_auto_commit = settings.Setting(True)
    maxNodeSize = settings.Setting(50)
    minNodeSize = settings.Setting(8)
    selectionMode = settings.Setting(0)
    tabIndex = settings.Setting(0)
    showEdgeWeights = settings.Setting(False)
    relativeEdgeWidths = settings.Setting(False)
    invertNodeSize = settings.Setting(False)
    markDistance = settings.Setting(1)
    markSearchString = settings.Setting("")
    markNBest = settings.Setting(1)
    markNConnections = settings.Setting(2)

    def __init__(self):
        super().__init__()
        #self.contextHandlers = {"": DomainContextHandler("", [ContextField("attributes", selected="node_label_attrs"), ContextField("attributes", selected="tooltipAttributes"), "color"])}

        self.view = GraphView(self)
        self.mainArea.layout().addWidget(self.view)

        self.graph_attrs = []

        self.acceptingEnterKeypress = False

        self.node_label_attrs = []
        self.tooltipAttributes = []
        self.searchStringTimer = QTimer(self)
        self.markInputItems = None
        self.node_color_attr = 0
        self.node_size_attr = 0

        self.nHighlighted = 0
        self.nSelected = 0
        self.verticesPerEdge = 0
        self.edgesPerVertex = 0

        self.lastVertexSizeColumn = ''
        self.lastColorColumn = ''
        self.lastLabelColumns = set()
        self.lastTooltipColumns = set()

        self.items_matrix = None
        self.number_of_nodes_label = 0
        self.number_of_edges_label = 0

        self.graph = None

        self.setMinimumWidth(600)

        self.tabs = gui.tabWidget(self.controlArea)
        self.displayTab = gui.createTabPage(self.tabs, "Display")
        self.markTab = gui.createTabPage(self.tabs, "Marking")

        def on_tab_changed(index):
            self.tabIndex = index
            self.set_selection_mode()
        self.tabs.currentChanged.connect(on_tab_changed)
        self.tabs.setCurrentIndex(self.tabIndex)

        ib = gui.widgetBox(self.displayTab, "Info")
        gui.label(ib, self, "Nodes: %(number_of_nodes_label)i (%(verticesPerEdge).2f per edge)")
        gui.label(ib, self, "Edges: %(number_of_edges_label)i (%(edgesPerVertex).2f per node)")

        box = gui.widgetBox(self.displayTab, "Nodes")

        self.relayout_button = gui.button(box, self, 'Re-layout',
                                          callback=self.relayout, autoDefault=False)
        self.view.positionsChanged.connect(lambda _: self.progressbar.advance())
        def animationFinished():
            self.relayout_button.setEnabled(True)
            self.progressbar.finish()
        self.view.animationFinished.connect(animationFinished)

        self.colorCombo = gui.comboBox(
            box, self, "node_color_attr", label='Color:',
            orientation='horizontal', callback=self.set_node_colors)

        self.invertNodeSizeCheck = self.maxNodeSizeSpin = QWidget()  # Forward declaration
        self.nodeSizeCombo = gui.comboBox(
            box, self, "node_size_attr",
            label='Size:',
            orientation='horizontal',
            callback=self.set_node_sizes)
        hb = gui.widgetBox(box, orientation="horizontal")
        hb.layout().addStretch(1)
        self.minNodeSizeSpin = gui.spin(
            hb, self, "minNodeSize", 1, 50, step=1, label="Min:",
            callback=self.set_node_sizes)
        self.minNodeSizeSpin.setValue(8)
        gui.separator(hb)
        self.maxNodeSizeSpin = gui.spin(
            hb, self, "maxNodeSize", 10, 200, step=5, label="Max:",
            callback=self.set_node_sizes)
        self.maxNodeSizeSpin.setValue(50)
        gui.separator(hb)
        self.invertNodeSizeCheck = gui.checkBox(
            hb, self, "invertNodeSize", "Invert",
            callback=self.set_node_sizes)

        hb = gui.widgetBox(self.displayTab, box="Node labels | tooltips",
                           orientation="horizontal", addSpace=False)
        self.attListBox = gui.listBox(
            hb, self, "node_label_attrs", "graph_attrs",
            selectionMode=QListWidget.MultiSelection,
            sizeHint=QSize(100, 100),
            callback=self._on_node_label_attrs_changed)
        self.tooltipListBox = gui.listBox(
            hb, self, "tooltipAttributes", "graph_attrs",
            selectionMode=QListWidget.MultiSelection,
            sizeHint=QSize(100, 100),
            callback=self._clicked_tooltip_lstbox)

        eb = gui.widgetBox(self.displayTab, "Edges", orientation="vertical")
        self.checkbox_relative_edges = gui.checkBox(
            eb, self, 'relativeEdgeWidths', 'Relative edge widths',
            callback=self.set_edge_sizes)
        self.checkbox_show_weights = gui.checkBox(
            eb, self, 'showEdgeWeights', 'Show edge weights',
            callback=self.set_edge_labels)

        ib = gui.widgetBox(self.markTab, "Info", orientation="vertical")
        gui.label(ib, self, "Nodes: %(number_of_nodes_label)i")
        gui.label(ib, self, "Selected: %(nSelected)i")
        gui.label(ib, self, "Highlighted: %(nHighlighted)i")
        def on_selection_change():
            self.nSelected = len(self.view.getSelected())
            self.nHighlighted = len(self.view.getHighlighted())
            self.set_selection_mode()
            self.commit()
        self.view.selectionChanged.connect(on_selection_change)

        ib = gui.widgetBox(self.markTab, "Highlight nodes ...")
        ribg = gui.radioButtonsInBox(ib, self, "selectionMode", callback=self.set_selection_mode)
        gui.appendRadioButton(ribg, "None")
        gui.appendRadioButton(ribg, "... whose attributes contain:")
        self.ctrlMarkSearchString = gui.lineEdit(gui.indentedBox(ribg), self, "markSearchString", callback=self._set_search_string_timer, callbackOnType=True)
        self.searchStringTimer.timeout.connect(self.set_selection_mode)

        gui.appendRadioButton(ribg, "... neighbours of selected, ≤ N hops away")
        ib = gui.indentedBox(ribg, orientation=0)
        self.ctrlMarkDistance = gui.spin(ib, self, "markDistance", 1, 100, 1, label="Hops:",
            callback=lambda: self.set_selection_mode(SelectionMode.NEIGHBORS))
        ib.layout().addStretch(1)
        gui.appendRadioButton(ribg, "... with at least N connections")
        gui.appendRadioButton(ribg, "... with at most N connections")
        ib = gui.indentedBox(ribg, orientation=0)
        self.ctrlMarkNConnections = gui.spin(ib, self, "markNConnections", 0, 1000000, 1, label="Connections:",
            callback=lambda: self.set_selection_mode(SelectionMode.AT_MOST_N if self.selectionMode == SelectionMode.AT_MOST_N else SelectionMode.AT_LEAST_N))
        ib.layout().addStretch(1)
        gui.appendRadioButton(ribg, "... with more connections than any neighbor")
        gui.appendRadioButton(ribg, "... with more connections than average neighbor")
        gui.appendRadioButton(ribg, "... with most connections")
        ib = gui.indentedBox(ribg, orientation=0)
        self.ctrlMarkNumber = gui.spin(ib, self, "markNBest", 1, 1000000, 1,
                                       label="Number of nodes:",
                                       callback=lambda: self.set_selection_mode(SelectionMode.MOST_CONN))
        ib.layout().addStretch(1)
        self.markInputRadioButton = gui.appendRadioButton(ribg, "... given in the ItemSubset input signal")
        self.markInput = 0
        ib = gui.indentedBox(ribg)
        self.markInputCombo = gui.comboBox(ib, self, 'markInput',
                                           callback=lambda: self.set_selection_mode(SelectionMode.FROM_INPUT))
        self.markInputRadioButton.setEnabled(False)

        gui.auto_commit(ribg, self, 'do_auto_commit', 'Output changes')
        self.markTab.layout().addStretch(1)

        self.set_graph(None)
        self.set_selection_mode()

    def commit(self):
        self.send_data()

    def set_items_distance_matrix(self, matrix):
        assert matrix is None or isinstance(matrix, Orange.misc.DistMatrix)
        self.items_matrix = matrix
        self.relayout()

    def _set_search_string_timer(self):
        self.selectionMode = SelectionMode.SEARCH
        self.searchStringTimer.stop()
        self.searchStringTimer.start(300)

    def switchTab(self, index=None):
        index = index or self.tabs.currentIndex()
        curTab = self.tabs.widget(index)
        self.acceptingEnterKeypress = False
        if curTab == self.markTab and self.selectionMode != SelectionMode.NONE:
            self.acceptingEnterKeypress = True

    @non_reentrant
    def set_selection_mode(self, selectionMode=None):
        self.searchStringTimer.stop()
        selectionMode = self.selectionMode = selectionMode or self.selectionMode
        self.switchTab()
        if (self.graph is None or
            self.tabs.widget(self.tabs.currentIndex()) != self.markTab):
            return

        if selectionMode == SelectionMode.NONE:
            self.view.setHighlighted([])
        elif selectionMode == SelectionMode.SEARCH:
            table, txt = self.graph.items(), self.markSearchString.lower()
            if not table or not txt: return
            toMark = set(i for i, instance in enumerate(table)
                         if txt in " ".join(map(str, instance.list)).lower())
            self.view.setHighlighted(toMark)
        elif selectionMode == SelectionMode.NEIGHBORS:
            selected = set(self.view.getSelected())
            neighbors = selected.copy()
            for _ in range(self.markDistance):
                for neigh in list(neighbors):
                    neighbors |= set(self.graph[neigh].keys())
            neighbors -= selected
            self.view.setHighlighted(neighbors)
        elif selectionMode == SelectionMode.AT_LEAST_N:
            self.view.setHighlighted(
                set(node for node, degree in self.graph.degree().items()
                    if degree >= self.markNConnections))
        elif selectionMode == SelectionMode.AT_MOST_N:
            self.view.setHighlighted(
                set(node for node, degree in self.graph.degree().items()
                    if degree <= self.markNConnections))
        elif selectionMode == SelectionMode.ANY_NEIGH:
            self.view.setHighlighted(
                set(node for node, degree in self.graph.degree().items()
                    if degree > max(self.graph.degree(self.graph[node]).values(), default=0)))
        elif selectionMode == SelectionMode.AVG_NEIGH:
            self.view.setHighlighted(
                set(node for node, degree in self.graph.degree().items()
                    if degree > np.nan_to_num(np.mean(list(self.graph.degree(self.graph[node]).values())))))
        elif selectionMode == SelectionMode.MOST_CONN:
            degrees = np.array(sorted(self.graph.degree().items(), key=lambda i: i[1], reverse=True))
            cut_ind = max(1, min(self.markNBest, self.graph.number_of_nodes()))
            cut_degree = degrees[cut_ind - 1, 1]
            toMark = set(degrees[degrees[:, 1] >= cut_degree, 0])
            self.view.setHighlighted(toMark)
        elif selectionMode == SelectionMode.FROM_INPUT:
            var = self.markInputCombo.currentText()
            tomark = {}
            if self.markInputItems:
                if var == 'ID':
                    values = {x.id for x in self.markInputItems}
                    tomark = {x for x in self.graph.nodes()
                              if self.graph.items()[x].id in values}
                else:
                    clean = lambda s: str(s).strip().upper()
                    values = {clean(x[var]) for x in self.markInputItems}
                    tomark = {x for x in self.graph.nodes()
                              if clean(self.graph.items()[x][var]) in values}
            self.view.setHighlighted(tomark)

    def keyReleaseEvent(self, ev):
        """On Enter, expand the selected set with the highlighted"""
        if (not self.acceptingEnterKeypress or
            ev.key() not in (Qt.Key_Return, Qt.Key_Enter)):
            super().keyReleaseEvent(ev)
            return
        highlighted = self.view.getHighlighted()
        self.view.setSelected(highlighted, extend=True)
        self.view.setHighlighted([])
        self.set_selection_mode()

    def save_network(self):
        # TODO: this was never reviewed since Orange2
        if self.view is None or self.graph is None:
            return

        filename = QFileDialog.getSaveFileName(
            self, 'Save Network', '',
            'NetworkX graph as Python pickle (*.gpickle)\n'
            'Pajek network (*.net)\n'
            'GML network (*.gml)')
        if filename:
            _, ext = os.path.splitext(filename)
            if not ext: filename += ".net"
            items = self.graph.items()
            for i in range(self.graph.number_of_nodes()):
                graph_node = self.graph.node[i]
                plot_node = self.networkCanvas.networkCurve.nodes()[i]

                if items is not None:
                    ex = items[i]
                    if 'x' in ex.domain: ex['x'] = plot_node.x()
                    if 'y' in ex.domain: ex['y'] = plot_node.y()

                graph_node['x'] = plot_node.x()
                graph_node['y'] = plot_node.y()

            network.readwrite.write(self.graph, filename)

    def send_data(self):
        if not self.graph:
            for output in Output.all:
                self.send(output, None)
            return
        selected = self.view.getSelected()
        self.send(Output.SUBGRAPH,
                  self.graph.subgraph(selected) if selected else None)
        self.send(Output.DISTANCE,
                  self.items_matrix.submatrix(sorted(selected)) if self.items_matrix is not None and selected else None)
        items = self.graph.items()
        if not items:
            self.send(Output.SELECTED, None)
            self.send(Output.HIGHLIGHTED, None)
            self.send(Output.REMAINING, None)
        else:
            highlighted = self.view.getHighlighted()
            self.send(Output.SELECTED, items[sorted(selected), :] if selected else None)
            self.send(Output.HIGHLIGHTED, items[sorted(highlighted), :] if highlighted else None)
            remaining = sorted(set(self.graph) - set(selected) - set(highlighted))
            self.send(Output.REMAINING, items[remaining, :] if remaining else None)

    def _set_combos(self):
        self._clear_combos()
        self.graph_attrs = self.graph.items_vars()
        lastLabelColumns = self.lastLabelColumns
        lastTooltipColumns = self.lastTooltipColumns

        for var in self.graph_attrs:
            if var.is_discrete or var.is_continuous:
                self.colorCombo.addItem(gui.attributeIconDict[gui.vartype(var)], var.name, var)

            if var.is_continuous:
                self.nodeSizeCombo.addItem(gui.attributeIconDict[gui.vartype(var)], var.name, var)
            elif var.is_string:
                try: value = self.graph.items()[0][var].value
                except (IndexError, TypeError): pass
                else:
                    # can value be a list?
                    if len(value.split(',')) > 1:
                        self.nodeSizeCombo.addItem(gui.attributeIconDict[gui.vartype(var)], var.name, var)

        self.nodeSizeCombo.setDisabled(not self.graph_attrs)
        self.colorCombo.setDisabled(not self.graph_attrs)

        for i in range(self.nodeSizeCombo.count()):
            if self.lastVertexSizeColumn == \
                    self.nodeSizeCombo.itemText(i):
                self.node_size_attr = i
                self.set_node_sizes()
                break

        for i in range(self.colorCombo.count()):
            if self.lastColorColumn == self.colorCombo.itemText(i):
                self.node_color_attr = i
                self.set_node_colors()
                break

        for i in range(self.attListBox.count()):
            if str(self.attListBox.item(i).text()) in lastLabelColumns:
                self.attListBox.item(i).setSelected(True)
        self._on_node_label_attrs_changed()

        for i in range(self.tooltipListBox.count()):
            if (self.tooltipListBox.item(i).text() in lastTooltipColumns or
                not lastTooltipColumns):
                self.tooltipListBox.item(i).setSelected(True)

        self._clicked_tooltip_lstbox()

        self.lastLabelColumns = lastLabelColumns
        self.lastTooltipColumns = lastTooltipColumns

    def _clear_combos(self):
        self.graph_attrs = []

        self.colorCombo.clear()
        self.nodeSizeCombo.clear()

        self.colorCombo.addItem('(none)', None)
        self.nodeSizeCombo.addItem("(uniform)")

    def set_graph_none(self):
        self.graph = None
        self.graph_base = None
        self._clear_combos()
        self.number_of_nodes_label = 0
        self.number_of_edges_label = 0
        self.verticesPerEdge = 0
        self.edgesPerVertex = 0
        self._items = None
        self.view.set_graph(None)

    def set_graph(self, graph):
        if not graph:
            return self.set_graph_none()
        if graph.number_of_nodes() < 2:
            self.set_graph_none()
            self.information('I\'m not really in a mood to visualize just one node. Try again tomorrow.')
            return
        self.information()

        all_edges_equal = bool(1 == len(set(w for u,v,w in graph.edges_iter(data='weight'))))
        self.checkbox_show_weights.setEnabled(not all_edges_equal)
        self.checkbox_relative_edges.setEnabled(not all_edges_equal)

        self.graph_base = graph
        self.graph = graph.copy()
        # Set items table from the separate signal
        if self._items: self.set_items(self._items)

        self.view.set_graph(self.graph, relayout=False)

        # Set labels
        self.number_of_nodes_label = self.graph.number_of_nodes()
        self.number_of_edges_label = self.graph.number_of_edges()
        self.verticesPerEdge = self.graph.number_of_nodes() / max(1, self.graph.number_of_edges())
        self.edgesPerVertex = self.graph.number_of_edges() / max(1, self.graph.number_of_nodes())

        self._set_combos()
        if self.graph.number_of_nodes() + self.graph.number_of_edges() > 30000:
            self.set_graph_none()
            self.error('Network is too large to visualize. Sorry.')
            return
        self.error()

        self.set_selection_mode()
        self.relayout()

    def set_items(self, items=None):
        self._items = items
        if items is None:
            return self.set_graph(self.graph_base)
        if not self.graph:
            self.warning('No graph found!')
            return
        self.warning()
        if len(items) != self.graph.number_of_nodes():
            self.error('Items table must have one instance for each network node.')
            return
        self.error()
        self.graph.set_items(items)
        self._set_combos()

    def set_marking_items(self, items):
        self.markInputCombo.clear()
        self.markInputRadioButton.setEnabled(False)
        self.markInputItems = items

        self.warning()

        if items is None:
            return

        if self.graph is None or self.graph.items() is None:
            self.warning('No graph provided or no items attached to the graph.')
            return

        graph_items = self.graph.items()
        domain = graph_items.domain

        if len(items) > 0:
            commonVars = (set(x.name for x in chain(items.domain.variables,
                                                    items.domain.metas))
                          & set(x.name for x in chain(domain.variables,
                                                      domain.metas)))

            self.markInputCombo.addItem(gui.attributeIconDict[gui.vartype(DiscreteVariable())], "ID")

            for var in commonVars:
                orgVar, mrkVar = domain[var], items.domain[var]

                if type(orgVar) == type(mrkVar) == StringVariable:
                    self.markInputCombo.addItem(gui.attributeIconDict[gui.vartype(orgVar)], orgVar.name)

            self.markInputRadioButton.setEnabled(True)

    def relayout(self):
        if self.graph is None or self.graph.number_of_nodes() <= 1:
            return
        self.progressbar = gui.ProgressBar(self, FR_ITERATIONS)

        distmatrix = self.items_matrix
        if distmatrix is not None and distmatrix.shape[0] != self.graph.number_of_nodes():
            self.warning(17, "Distance matrix size doesn't match the number of network nodes. Not using it.")
            distmatrix = None
        self.warning(17)

        self.relayout_button.setDisabled(True)
        self.view.relayout(randomize=False, weight=distmatrix)

    def _on_node_label_attrs_changed(self):
        if not self.graph: return
        attributes = self.lastLabelColumns = [self.graph_attrs[i] for i in self.node_label_attrs]
        if attributes:
            table = self.graph.items()
            if not table: return
            for i, node in enumerate(self.view.nodes):
                text = ', '.join(map(str, table[i, attributes][0].list))
                node.setText(text)
        else:
            for node in self.view.nodes:
                node.setText('')

    def _clicked_tooltip_lstbox(self):
        if not self.graph: return
        attributes = self.lastTooltipColumns = [self.graph_attrs[i] for i in self.tooltipAttributes]
        if attributes:
            table = self.graph.items()
            if not table: return
            assert self.view.nodes
            for i, node in enumerate(self.view.nodes):
                node.setTooltip(lambda row=i, attributes=attributes, table=table:
                    '<br>'.join('<b>{.name}:</b> {}'.format(i[0], str(i[1]).replace('<', '&lt;'))
                                for i in zip(attributes, table[row, attributes][0].list))
                )
        else:
            for node in self.view.nodes:
                node.setTooltip(None)

    def set_edge_labels(self):
        if self.showEdgeWeights:
            weights = (str(w or '') for u, v, w in self.graph.edges_iter(data='weight'))
        else:
            weights = ('' for i in range(self.graph.number_of_edges()))
        for edge, weight in zip(self.view.edges, weights):
            edge.setText(weight)

    def set_node_colors(self):
        if not self.graph: return
        self.lastColorColumn = self.colorCombo.currentText()
        attribute = self.colorCombo.itemData(self.colorCombo.currentIndex())
        assert not attribute or isinstance(attribute, Orange.data.Variable)
        if not attribute:
            for node in self.view.nodes:
                node.setColor(None)
            return
        table = self.graph.items()
        if not table: return
        if attribute in table.domain.class_vars:
            values = table[:, attribute].Y
            if values.ndim > 1:
                values = values.T
        elif attribute in table.domain.metas:
            values = table[:, attribute].metas[:, 0]
        elif attribute in table.domain.attributes:
            values = table[:, attribute].X[:, 0]
        else: raise RuntimeError("Shouldn't be able to select this column")
        if attribute.is_continuous:
            colors = CONTINUOUS_PALETTE[scale(values)]
        elif attribute.is_discrete:
            DISCRETE_PALETTE = ColorPaletteGenerator(len(attribute.values))
            colors = DISCRETE_PALETTE[values]
        for node, color in zip(self.view.nodes, colors):
            node.setColor(color)

    def set_node_sizes(self):
        attribute = self.nodeSizeCombo.itemData(self.nodeSizeCombo.currentIndex())
        depending_widgets = (self.invertNodeSizeCheck, self.maxNodeSizeSpin)
        for w in depending_widgets:
            w.setDisabled(not attribute)
        if not self.graph: return
        table = self.graph.items()
        if not table: return
        if attribute in table.domain.class_vars:
            values = table[:, attribute].Y
            if values.ndim > 1:
                values = values.T
        elif attribute in table.domain.metas:
            values = table[:, attribute].metas[:, 0]
        elif attribute in table.domain.attributes:
            values = table[:, attribute].X[:, 0]
        else:
            for node in self.view.nodes:
                node.setSize(self.minNodeSize)
            return
        values = np.array(values)
        if self.invertNodeSize:
            values += np.nanmin(values) + 1
            values = 1/values
        nodemin, nodemax = np.nanmin(values), np.nanmax(values)
        if nodemin == nodemax:
            # np.polyfit borks on this condition
            sizes = (self.minNodeSize for i in range(len(self.view.nodes)))
        else:
            k, n = np.polyfit([nodemin, nodemax],
                              [self.minNodeSize, self.maxNodeSize], 1)
            sizes = values * k + n
            sizes[np.isnan(sizes)] = np.nanmean(sizes)
        for node, size in zip(self.view.nodes, sizes):
            node.setSize(size)

    def set_edge_sizes(self):
        if not self.graph: return
        if self.relativeEdgeWidths:
            widths = [self.graph.edge[u][v].get('weight', 1)
                      for u, v in self.graph.edges()]
            widths = scale(widths, .7, 8)
        else:
            widths = (.7 for i in range(self.graph.number_of_edges()))
        for edge, width in zip(self.view.edges, widths):
            edge.setSize(width)

    def sendReport(self):
        self.reportSettings("Graph data",
                            [("Number of vertices", self.graph.number_of_nodes()),
                             ("Number of edges", self.graph.number_of_edges()),
                             ("Vertices per edge", "%.3f" % self.verticesPerEdge),
                             ("Edges per vertex", "%.3f" % self.edgesPerVertex),
                             ])
        if self.node_color_attr or self.node_size_attr or self.node_label_attrs:
            self.reportSettings("Visual settings",
                                [self.node_color_attr and ("Vertex color", self.colorCombo.currentText()),
                                 self.node_size_attr and ("Vertex size", str(self.nodeSizeCombo.currentText()) + " (inverted)" if self.invertNodeSize else ""),
                                 self.node_label_attrs and ("Labels", ", ".join(self.graph_attrs[i].name for i in self.node_label_attrs)),
                                ])
        self.reportSection("Graph")
        self.reportImage(self.view)


if __name__ == "__main__":
    import sys
    a = QApplication(sys.argv)
    ow = OWNxExplorer()
    ow.show()

    def setNetwork(signal, data, id=None):
        if signal == 'Network':
            ow.set_graph(data)
        #if signal == 'Items':
        #    ow.set_items(data)

    import OWNxFile
    from os.path import join, dirname
    owFile = OWNxFile.OWNxFile()
    owFile.send = setNetwork
    owFile.openNetFile(join(dirname(dirname(__file__)), 'networks', 'leu_by_genesets.net'))
    #~ owFile.openFile(join(dirname(dirname(__file__)), 'networks', 'airtraffic.net'))
    #~ owFile.openFile(join(dirname(dirname(__file__)), 'networks', 'lastfm.net'))
    #~ owFile.show()
    #~ owFile.selectNetFile(0)

    a.exec_()
    ow.saveSettings()
    owFile.saveSettings()



"""
.. index:: Reading and Writing Networks

.. index::
   single: Network; Reading and Writing Networks

****************************
Reading and Writing Networks
****************************

Use methods in the :obj:`Orange.network.readwrite` module to read and write
networks in Orange. If you must use the original NetworkX read/write methods
(for some reason), do not forget to cast the network object (see
Orange.network.readwrite._wrap method).

"""

import os
import itertools
import tempfile
import gzip

import networkx as nx
import networkx.readwrite.pajek as rwpajek
import networkx.readwrite.gml as rwgml
import networkx.readwrite.gpickle as rwgpickle

import Orange

from .network import Graph, DiGraph, MultiGraph, MultiDiGraph


SUPPORTED_READ_EXTENSIONS = ['.net', '.pajek', '.gml', '.gpickle', '.gz']
SUPPORTED_WRITE_EXTENSIONS = ['.net', '.pajek', '.gml', '.gpickle']


def _wrap(g):
    for base, new in [(nx.Graph, Graph),
                      (nx.DiGraph, DiGraph),
                      (nx.MultiGraph, MultiGraph),
                      (nx.MultiDiGraph, MultiDiGraph)]:
        if isinstance(g, base):
            return g if isinstance(g, new) else new(g, name=g.name)
    return g

def _add_doc(myclass, nxclass):
    tmp = nxclass.__doc__.replace('nx.write', 'Orange.network.readwrite.write')
    tmp = tmp.replace('nx.read', 'Orange.network.readwrite.read')
    tmp = tmp.replace('nx', 'Orange.network.nx')
    myclass.__doc__ += tmp

def _is_string_like(obj): # from John Hunter, types-free version
    """Check if obj is string."""
    try:
        obj + ''
    except (TypeError, ValueError):
        return False
    return True

def _get_fh(path, mode='r'):
    """Return a file handle for given path.

    Path can be a string or a file handle.

    Attempt to uncompress/compress files ending in '.gz' and '.bz2'.

    """
    if _is_string_like(path):
        if path.endswith('.gz'):
            import gzip
            fh = gzip.open(path, mode=mode)
        elif path.endswith('.bz2'):
            import bz2
            fh = bz2.BZ2File(path, mode=mode)
        else:
            fh = open(path, mode=mode)
    elif hasattr(path, 'read'):
        fh = path
    else:
        raise ValueError('path must be a string or file handle')
    return fh

def _make_str(t):
    """Return the string representation of t."""
    if _is_string_like(t): return t
    return str(t)

def _check_network_dir(p):
    if type(p) == str:
        if not os.path.isfile(p):
            path = Orange.data.io.find_file(os.path.split(p)[1])

            if os.path.isfile(path):
                return path

            raise OSError('File %s does not exist.' % p)

    return p

def graph_to_table(G):
    """Builds a Data Table from node values."""
    if G.number_of_nodes() > 0:
        features = list(set(itertools.chain.from_iterable(node.keys() for node in G.node.values())))
        data = [[node.get(f).replace('\t', ' ') if isinstance(node.get(f, 1), str) else str(node.get(f, '?'))
                 for f in features]
                for node in G.node.values()]
        fp = tempfile.NamedTemporaryFile('wt', suffix='.txt', delete=False)
        fp.write('\n'.join('\t'.join(line) for line in [features] + data))
        fp.close()
        table = Orange.data.Table(fp.name)
        os.unlink(fp.name)

    return table

def read(path, encoding='UTF-8', auto_table=0):
    """Read graph in any of the supported file formats (.gpickle, .net, .gml).
    The parser is chosen based on the file extension.

    :param path: File or filename to write.
    :type path: string

    Return the network of type :obj:`Orange.network.Graph`,
    :obj:`Orange.network.DiGraph`, :obj:`Orange.network.Graph` or
    :obj:`Orange.network.DiGraph`.

    """
    path = _check_network_dir(path)
    _, ext = os.path.splitext(path.lower())
    if not ext in SUPPORTED_READ_EXTENSIONS:
        raise ValueError('Extension %s is not supported.' % ext)

    if ext in ('.net', '.pajek'):
        return read_pajek(path, encoding, auto_table=auto_table)

    if ext == '.gml':
        return read_gml(path, encoding, auto_table=auto_table)

    if ext == '.gpickle':
        return read_gpickle(path, auto_table=auto_table)

    if ext == '.gz' and path[-6:] == 'txt.gz':
        return read_txtgz(path)

def write(G, path, encoding='UTF-8'):
    """Write graph in any of the supported file formats (.gpickle, .net, .gml).
    The file format is chosen based on the file extension.

    :param G: A Orange graph.
    :type G: Orange.network.Graph

    :param path: File or filename to write.
    :type path: string

    """
    _, ext = os.path.splitext(path.lower())
    if not ext in SUPPORTED_WRITE_EXTENSIONS:
        raise ValueError('Extension %s is not supported. Use %s.' % (ext, ', '.join(supported)))

    if ext in ('.net', '.pajek'):
        write_pajek(G, path, encoding)

    if ext == '.gml':
        write_gml(G, path)

    if ext == '.gpickle':
        write_gpickle(G, path)

    if G.items() is not None:
        G.items().save(root + '_items.tab')

    if G.links() is not None:
        G.links().save(root + '_links.tab')

def read_gpickle(path, auto_table=False):
    """NetworkX read_gpickle method and wrap graph to Orange network.

    """

    path = _check_network_dir(path)

    G = _wrap(rwgpickle.read_gpickle(path))
    if auto_table:
        G.set_items(graph_to_table(G))
    return G

#~ _add_doc(read_gpickle, rwgpickle.read_gpickle)

def write_gpickle(G, path):
    """NetworkX write_gpickle method.

    """

    rwgpickle.write_gpickle(G, path)

_add_doc(write_gpickle, rwgpickle.write_gpickle)

def read_pajek(path, encoding='UTF-8', project=False, auto_table=False):
    """Reimplemented method for reading Pajek files; written in
    C++ for maximum performance.

    :param path: File or file name to write.
    :type path: string

    :param encoding: Encoding of input text file, default 'UTF-8'.
    :type encoding: string

    :param project: Determines whether the input file is a Pajek project file,
        possibly containing multiple networks and other data. If :obj:`True`,
        a list of networks is returned instead of just a network. Default is
        :obj:`False`.
    :type project: boolean.

    Return the network (or a list of networks if project=:obj:`True`) of type
    :obj:`Orange.network.Graph` or :obj:`Orange.network.DiGraph`.


    Examples

    >>> G=Orange.network.nx.path_graph(4)
    >>> Orange.network.readwrite.write_pajek(G, "test.net")
    >>> G=Orange.network.readwrite.read_pajek("test.net")

    To create a Graph instead of a MultiGraph use

    >>> G1=Orange.network.Graph(G)

    References

    See http://vlado.fmf.uni-lj.si/pub/networks/pajek/doc/draweps.htm
    for format information.

    """

    path = _check_network_dir(path)
    G = _wrap(rwpajek.read_pajek(path))

    # Additionally read values into Table; needed to get G nodes properly sorted
    # (Consult OWNxFile.readDataFile(), orangeom.GraphLayout.readPajek(), and the Pajek format spec)
    import shlex, numpy as np
    rows, metas, remapping = [], [], {}
    with open(path) as f:
        for line in f:
            if line.lower().startswith('*vertices'):
                nvertices = int(line.split()[1])
                break
        # Read vertices lines
        for line in f:
            parts = shlex.split(line)[:4]
            if len(parts) == 1:
                i = label = parts[0]
            elif len(parts) == 2:
                i, label = parts
                metas.append((label,))
            elif len(parts) == 4:
                i, label, x, y = parts
                # The format specification was never set in stone, it seems
                try:
                    x, y = float(x), float(y)
                except ValueError:
                    metas.append((label, x, y))
                else:
                    rows.append((x, y))
                    metas.append((label,))
            i = int(i) - 1  # -1 because pajek is 1-indexed
            remapping[label] = i
            nvertices -= 1
            if not nvertices: break
    from Orange.data import Domain, Table, ContinuousVariable, StringVariable
    # Construct x-y-label table (added in OWNxFile.readDataFile())
    table = None
    vars = [ContinuousVariable('x'), ContinuousVariable('y')] if rows else []
    meta_vars = [StringVariable('label ' + str(i)) for i in range(len(metas[0]) if metas else 0)]
    if rows or metas:
        domain = Domain(vars, metas=meta_vars)
        table = Table.from_numpy(domain,
                                 np.array(rows, dtype=float).reshape(len(metas),
                                                                     len(rows[0]) if rows else 0),
                                 metas=np.array(metas, dtype=str))
    if table is not None and auto_table:
        G.set_items(table)
    # Relabel nodes to integers, sorted by appearance
    for node in G.node:
        G.node[node]['label'] = node
    nx.relabel_nodes(G, remapping, copy=False)
    assert not table or len(table) == G.number_of_nodes(), 'There was a bug in NetworkX. Please update to git if need be'
    return G

def write_pajek(G, path, encoding='UTF-8'):
    """A copy & paste of NetworkX's function with some bugs fixed (call the new
    generate_pajek).

    """

    fh=_get_fh(path, 'wb')
    for line in generate_pajek(G):
        line+='\n'
        fh.write(line.encode(encoding))

_add_doc(write_pajek, rwpajek.write_pajek)

def parse_pajek(lines):
    """Parse string in Pajek file format. See read_pajek for usage examples.

    :param lines: a string of network data in Pajek file format.
    :type lines: string

    """

    return read_pajek(lines)


def generate_pajek(G):
    """A copy & paste of NetworkX's function with some bugs fixed (generate
    one line per object: vertex, edge, arc. Do not add one per entry in data
    dictionary).

    Generate lines in Pajek graph format.

    :param G: A Orange graph.
    :type G: Orange.network.Graph

    References

    See http://vlado.fmf.uni-lj.si/pub/networks/pajek/doc/draweps.htm
    for format information.

    """

    if G.name=='':
        name='NetworkX'
    else:
        name=G.name
    yield '*network %s'%name

    # write nodes with attributes
    yield '*vertices %s'%(G.order())
    nodes = G.nodes()
    # make dictionary mapping nodes to integers
    nodenumber=dict(zip(nodes,range(1,len(nodes)+1)))
    for n in nodes:
        na=G.node.get(n,{})
        x=na.get('x',0.0)
        y=na.get('y',0.0)
        id=int(na.get('id',nodenumber[n]))
        nodenumber[n]=id
        shape=na.get('shape','ellipse')
        s = ' '.join(map(_make_str,(id,n,x,y,shape)))
        for k,v in na.items():
            if k != 'x' and k != 'y':
                s += ' %s %s'%(k,v)
        yield s

    # write edges with attributes
    if G.is_directed():
        yield '*arcs'
    else:
        yield '*edges'
    for u,v,edgedata in G.edges(data=True):
        d=edgedata.copy()
        value=d.pop('weight',1.0) # use 1 as default edge value
        s = ' '.join(map(_make_str,(nodenumber[u],nodenumber[v],value)))
        for k,v in d.items():
            if not _is_string_like(v):
                v = repr(v)
            # add quotes to any values with a blank space
            if " " in v:
                v="\"%s\"" % v.replace('"', r'\"')
            s += ' %s %s'%(k,v)
        yield s


#_add_doc(generate_pajek, rwpajek.generate_pajek)

def read_gml(path, encoding='latin-1', relabel=False, auto_table=False):
    """NetworkX read_gml method and wrap graph to Orange network.

    """

    path = _check_network_dir(path)

    G = _wrap(rwgml.read_gml(path, encoding, relabel))
    if auto_table:
        G.set_items(graph_to_table(G))
    return G

_add_doc(read_gml, rwgml.read_gml)

def write_gml(G, path):
    """NetworkX write_gml method.

    """

    rwgml.write_gml(G, path)

def read_txtgz(path):
    f = gzip.open(path, 'rb')
    content = f.read()
    f.close()

    content = content.splitlines()
    comments = (line for line in content if line.strip().startswith(b'#'))
    content = (line for line in content if not line.strip().startswith(b'#'))

    if b"directed graph" in b''.join(comments).lower():
        G = DiGraph()
    else:
        G = Graph()

    G.add_edges_from([int(node) for node in coors.strip().split(b'\t')]
                      for coors in content
                      if len(coors.strip().split(b'\t')) == 2)

    return G

_add_doc(write_gml, rwgml.write_gml)



import sys
from PyQt4.QtGui import QApplication

appl = QApplication(sys.argv)

import Orange.data
import Orange.network
import OWNetExplorer

net = Orange.network.Network.read('musicians.net')
net.items = Orange.data.Table('musicians_items.tab')

ow = OWNetExplorer.OWNetExplorer()
ow.setGraph(net)

ow.show()
appl.exec_()


"""
.. index:: Network

*********
BaseGraph
*********

BaseGraph provides methods to work with additional data--describing nodes and
edges:

* items (:obj:`Orange.data.Table`) - information on nodes. Each row in the table corresponds to a node with ID matching row index.
* links (:obj:`Orange.data.Table`) - information on edges. Each row in the table corresponds to an edge. Two columns titled "u" and "v" should be specified in the table which contain indices of nodes on the given edge.

The BaseGraph class contains also other methods that are common to the four graph types.

.. autoclass:: Orange.network.BaseGraph
   :members:

***********
Graph Types
***********

The reference in this section is complemented with the original NetworkX
library reference. For a complete documentation please refer to the
`NetworkX docs <http://networkx.lanl.gov/reference/>`_. All methods from the
NetworkX package can be used for graph analysis and manipulation. For reading
and writing graphs refer to the Orange.network.readwrite docs.

Graph
=====

.. autoclass:: Orange.network.Graph
   :members:

DiGraph
=======

.. autoclass:: Orange.network.DiGraph
   :members:

MultiGraph
==========

.. autoclass:: Orange.network.MultiGraph
   :members:

MultiDiGraph
============

.. autoclass:: Orange.network.MultiDiGraph
   :members:

"""

import copy
import math
from itertools import chain

import numpy as np
import networkx as nx

import Orange

class MdsTypeClass():
    def __init__(self):
        self.componentMDS = 0
        self.exactSimulation = 1
        self.MDS = 2

MdsType = MdsTypeClass()

def _get_doc(doc):
    return doc.replace('nx.', 'Orange.network.') if doc else ''

class BaseGraph():
    """A collection of methods inherited by all graph types (:obj:`Graph`,
    :obj:`DiGraph`, :obj:`MultiGraph` and :obj:`MultiDiGraph`).

    """

    def __init__(self):
        self._items = None
        self._links = None

    def items(self):
        """Return the :obj:`Orange.data.Table` items with data about network
        nodes.

        """

        if self._items is not None and \
                        len(self._items) != self.number_of_nodes():
            print("Warning: items length does not match the number of nodes.")

        return self._items

    def set_items(self, items=None):
        """Set the :obj:`Orange.data.Table` items to the given data. Notice
        that the number of instances must match the number of nodes.

        """

        if items is not None:
            if not isinstance(items, Orange.data.Table):
                raise TypeError('items must be of type \'Orange.data.Table\'')
            if len(items) != self.number_of_nodes():
                print("Warning: items length must match the number of nodes.")

        self._items = items

    def links(self):
        """Return the :obj:`Orange.data.Table` links with data about network
        edges.

        """

        if self._links is not None \
                    and len(self._links) != self.number_of_edges():
            print("Warning: links length does not match the number of edges.")

        return self._links

    def set_links(self, links=None):
        """Set the :obj:`Orange.data.Table` links to the given data. Notice
        that the number of instances must match the number of edges.

        """

        if links is not None:
            if not isinstance(links, Orange.data.Table):
                raise TypeError('links must be of type \'Orange.data.Table\'')
            if len(links) != self.number_of_edges():
                print("Warning: links length must match the number of edges.")

        self._links = links

    def to_orange_network(self):
        """Convert the current network to >>Orange<< NetworkX standard. To use
        :obj:`Orange.network` in Orange widgets, set node IDs to be range
        [0, no_of_nodes - 1].

        """

        G = self.__class__()
        node_list = sorted(self.nodes())
        node_to_index = dict(zip(node_list, range(self.number_of_nodes())))
        index_to_node = dict(zip(range(self.number_of_nodes()), node_list))

        G.add_nodes_from(zip(range(self.number_of_nodes()), [copy.deepcopy(self.node[nid]) for nid in node_list]))
        G.add_edges_from(((node_to_index[u], node_to_index[v], copy.deepcopy(self.edge[u][v])) for u, v in self.edges()))

        for id in G.node.keys():
            G.node[id]['old_id'] = index_to_node[id]

        if self.items():
            G.set_items(self.items())

        if self.links():
            G.set_links(self.links())

        return G

    ### TODO: OVERRIDE METHODS THAT CHANGE GRAPH STRUCTURE, add warning prints

    def items_vars(self):
        """Return a list of features in the :obj:`Orange.data.Table` items."""
        if not self._items: return []
        return list(chain(self._items.domain.variables,
                          self._items.domain.metas))

    def links_vars(self):
        """Return a list of features in the :obj:`Orange.data.Table` links."""
        if not self._links: return []
        return [i for i in chain(self._items.domain.variables,
                                 self._items.domain.metas)
                if i.name not in ('u', 'v')]

    def subgraph(self, nbunch):
        G = super().subgraph(nbunch)
        G = G.to_orange_network()
        if self.items():
            items = self.items()[sorted(G.nodes()), :]
            G.set_items(items)
        return G

class Graph(BaseGraph, nx.Graph):
    """Bases: `NetworkX.Graph <http://networkx.lanl.gov/reference/classes.graph.html>`_,
    :obj:`Orange.network.BaseGraph`

    """

    def __init__(self, data=None, name='', **attr):
        nx.Graph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)
        # TODO: _links

    __doc__ += _get_doc(nx.Graph.__doc__)
    __init__.__doc__ = _get_doc(nx.Graph.__init__.__doc__)

class DiGraph(BaseGraph, nx.DiGraph):
    """Bases: `NetworkX.DiGraph <http://networkx.lanl.gov/reference/classes.digraph.html>`_,
    :obj:`Orange.network.BaseGraph`

    """


    def __init__(self, data=None, name='', **attr):
        nx.DiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.DiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.DiGraph.__init__.__doc__)

class MultiGraph(BaseGraph, nx.MultiGraph):
    """Bases: `NetworkX.MultiGraph <http://networkx.lanl.gov/reference/classes.multigraph.html>`_,
    :obj:`Orange.network.BaseGraph`

    """


    def __init__(self, data=None, name='', **attr):
        nx.MultiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.MultiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.MultiGraph.__init__.__doc__)

class MultiDiGraph(BaseGraph, nx.MultiDiGraph):
    """Bases: `NetworkX.MultiDiGraph <http://networkx.lanl.gov/reference/classes.multidigraph.html>`_,
    :obj:`Orange.network.BaseGraph`

    """


    def __init__(self, data=None, name='', **attr):
        nx.MultiDiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.MultiDiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.MultiDiGraph.__init__.__doc__)


class NxView(object):
    """Network View"""

    def __init__(self, **attr):
        self._network = None
        self._nx_explorer = None

    def set_nx_explorer(self, _nx_explorer):
        self._nx_explorer = _nx_explorer

    def init_network(self, graph):
        return graph

    def node_selection_changed(self):
        pass

    def update_network(self):
        if self._nx_explorer is not None and self._network is not None:
            subnet = self._network
            self._nx_explorer.change_graph(subnet)



#cython: boundscheck=False
#cython: wraparound=False
#cython: initializedcheck=False
#cython: cdivision=True
#cython: embedsignature=True
#cython: infer_types=False
#cython: language_level=3

"""
Fruchterman, T. M. J., Reingold, E. M.,

Graph Drawing by Force-directed Placement.

http://emr.cs.iit.edu/~reingold/force-directed.pdf
"""

cimport numpy as np
import numpy as np

from libc.math cimport exp, log, sqrt, fabs


ctypedef np.float64_t   double
ctypedef np.int32_t[:]  arr_i1_t
ctypedef double[:, :]   arr_f2_t
ctypedef double[:]      arr_f1_t


def fruchterman_reingold_layout(G,
                                dim=2,
                                k=None,
                                pos=None,
                                fixed=None,
                                iterations=50,
                                weight='weight',
                                callback=None):
    """
    Position nodes using Fruchterman-Reingold force-directed algorithm.

    The parameters are equal to those of networkx.spring_layout(), with the
    following exceptions:

    Parameters
    ----------
    weight: str or np.ndarray
        The string attribute of the graph's edges that represents edge weights,
        or a 2D distance matrix.
    callaback: callable
        A function accepting `pos` ndarray to call after each iteration.
        The algorithm is stopped if the return value is ``False``-ish.

    """
    if G is None or len(G) == 0: return {}
    if len(G) == 1: return {G.nodes()[0]: [.5, .5]}

    # Determine initial positions
    pos_arr = np.random.random((len(G), dim))
    if isinstance(pos, dict):
        for i, n in enumerate(G):
            if n in pos:
                pos_arr[i] = np.asarray(pos[n])
    elif isinstance(pos, np.ndarray):
        pos_arr = pos
    fixed = np.array(fixed or [], dtype=np.int32)

    # Prepare edges info as sparse COO matrix parts
    nodelist = sorted(G)
    index = dict(zip(nodelist, range(len(nodelist))))
    try:
        if isinstance(weight, str):
            Erow, Ecol, Edata = zip(*[(index[u], index[v], w or 0)  # 0 if None
                                      for u, v, w in G.edges_iter(nodelist, data=weight)])
        elif isinstance(weight, np.ndarray):
            Erow, Ecol, Edata = zip(*[(index[u], index[v], weight[u, v])
                                      for u, v in G.edges_iter(nodelist)])
        else: raise TypeError('weight must be str or ndarray')
        # Don't allow zero weights
        try: min_w = np.min([i for i in Edata if i])
        except ValueError: min_w = 1
        Edata = np.clip(Edata, min_w, np.inf)
    except ValueError:  # No edges
        Erow, Ecol, Edata = [], [], []
    Erow = np.asarray(Erow, dtype=np.int32)
    Ecol = np.asarray(Ecol, dtype=np.int32)
    Edata = np.asarray(Edata, dtype=np.float64)
    # Optimal distance between nodes
    k = k or 1 / sqrt(pos_arr.shape[0])
    # Run...
    pos = np.asarray(_fruchterman_reingold(Edata, Erow, Ecol,
                                           k, pos_arr, fixed,
                                           iterations, callback))
    return pos


cdef inline void diff(arr_f2_t pos,
                      Py_ssize_t i,
                      Py_ssize_t j,
                      arr_f1_t out) nogil:
    cdef Py_ssize_t d
    for d in range(out.shape[0]):
        out[d] = pos[i, d] - pos[j, d]


cdef inline double magnitude(arr_f1_t a) nogil:
    cdef:
        double result = 0
        Py_ssize_t i
    for i in range(a.shape[0]):
        result += a[i] * a[i]
    return sqrt(result)


cdef inline double magnitude2(arr_f2_t a, Py_ssize_t i) nogil:
    cdef:
        double result = 0
        Py_ssize_t j
    for j in range(a.shape[1]):
        result += a[i, j] * a[i, j]
    return sqrt(result)


cdef inline double _Fr(double k, double x) nogil:
    return k*k / x


cdef inline double _Fa(double k, double x) nogil:
    return x*x / k


cdef arr_f2_t _fruchterman_reingold(arr_f1_t Edata,  # COO matrix constituents
                                    arr_i1_t Erow,   #
                                    arr_i1_t Ecol,   #
                                    double k,
                                    arr_f2_t pos,
                                    arr_i1_t fixed,
                                    int iterations,
                                    callback):
    cdef:
        double GRAVITY = 20
        arr_f1_t temperature = (.15 *
                                exp(log(10./1000) / iterations)**np.arange(iterations))
        arr_f2_t disp = np.empty((pos.shape[0], pos.shape[1]))
        arr_f1_t delta = np.empty(pos.shape[1])
        double mag, adj, weight, temp
        Py_ssize_t row, col, i, j, d, iteration
        Py_ssize_t n_nodes = pos.shape[0]
        Py_ssize_t n_edges = Edata.shape[0]
        Py_ssize_t n_dim = pos.shape[1]
        int have_callback = bool(callback)
    with nogil:
        temperature[0] = .8; temperature[1] = .5; temperature[2] = .3
        for iteration in range(iterations):
            temp = temperature[iteration]
            disp[:, :] = 0
            # Repulsive forces
            for i in range(n_nodes):
                for j in range(n_nodes):
                    diff(pos, i, j, delta)
                    mag = magnitude(delta)
                    if mag == 0: continue
                    for d in range(n_dim):
                        disp[i, d] += delta[d] / mag * _Fr(k, mag)
            # Attractive forces
            for i in range(n_edges):
                row, col, weight = Erow[i], Ecol[i], Edata[i]
                diff(pos, row, col, delta)
                mag = magnitude(delta)
                if mag == 0: continue
                for d in range(n_dim):
                    adj = delta[d] / mag * weight * _Fa(k, mag)
                    disp[row, d] -= adj
                    disp[col, d] += adj
            # Gravity; tend toward center
            for i in range(n_nodes):
                mag = magnitude2(pos, i)
                for d in range(n_dim):
                    disp[i, d] -= k * GRAVITY * mag * pos[i, d]
            # Keep fixed nodes fixed
            for i in range(fixed.shape[0]):
                i = fixed[i]
                for d in range(n_dim):
                    disp[i, d] = 0
            # Limit the maximum displacement
            for i in range(n_nodes):
                mag = magnitude2(disp, i)
                if mag == 0: continue
                for d in range(n_dim):
                    pos[i, d] += disp[i, d] / mag * min(fabs(disp[i, d]), temp)
            # Optionally call back with the new positions
            if have_callback:
                with gil:
                    if not callback(np.asarray(pos)):
                        break
            # If temperature too cool, finish early
            if temp < .0005:
                break
    return pos



from PyQt4 import QtCore
from PyQt4.QtGui import *
from PyQt4.QtCore import *
import numpy as np
import networkx as nx
import sys

import Orange
from Orange.data import Table, Domain
from Orange.widgets import gui, widget
import orangecontrib.network as network


NODELEVEL = 0
GRAPHLEVEL = 1


class WorkerThread(QtCore.QThread):
    def __init__(self, receiver, name, label, type, algorithm):
        super().__init__()
        self.receiver = receiver
        self.name = name
        self.label = label
        self.type = type
        self.algorithm = algorithm

        self.stopped = 0
        self.result = None
        self.error = None
        self.is_terminated = False

    def run(self):
        try:
            self.result = self.algorithm(self.receiver.graph)
        except Exception as ex:
            self.result = None
            self.error = ex


class OWNxAnalysis(widget.OWWidget):
    name = 'Network Analysis'
    description = 'Statistical analysis of network data.'
    icon = 'icons/NetworkAnalysis.svg'
    priority = 6425

    resizing_enabled = False

    inputs = [("Network", network.Graph, 'set_graph'),
              ("Items", Orange.data.Table, 'set_items')]
    outputs = [("Network", network.Graph),
               ("Items", Orange.data.Table)]

    settingsList = [
        "auto_commit", "tab_index", "degree", "in_degree", "out_degree", "average_neighbor_degree",
        "clustering", "triangles", "square_clustering", "number_of_cliques",
        "degree_centrality", "in_degree_centrality", "out_degree_centrality",
        "closeness_centrality", "betweenness_centrality",
        "current_flow_closeness_centrality", "current_flow_betweenness_centrality",
        "approximate_current_flow_betweenness_centrality",
        "eigenvector_centrality", "eigenvector_centrality_numpy", "load_centrality",
        "core_number", "eccentricity", "closeness_vitality",

        "number_of_nodes", "number_of_edges", "average_degree", "density",
        "degree_assortativity_coefficient", "degree_pearson_correlation_coefficient",
        "degree_pearson_correlation_coefficient",
        "estrada_index", "graph_clique_number", "graph_number_of_cliques",
        "transitivity", "average_clustering", "number_connected_components",
        "number_strongly_connected_components", "number_weakly_connected_components",
        "number_attracting_components", "diameter", "radius", "average_shortest_path_length"
    ]
    # TODO: set settings

    want_main_area = False

    def __init__(self):
        super().__init__()

        self.methods = [
            ("number_of_nodes", True, "Number of nodes", GRAPHLEVEL, lambda G: G.number_of_nodes()),
            ("number_of_edges", True, "Number of edges", GRAPHLEVEL, lambda G: G.number_of_edges()),
            ("average_degree", True, "Average degree", GRAPHLEVEL, lambda G: np.average(list(G.degree().values()))),
            ("diameter", False, "Diameter", GRAPHLEVEL, nx.diameter),
            ("radius", False, "Radius", GRAPHLEVEL, nx.radius),
            ("average_shortest_path_length", False, "Average shortest path length", GRAPHLEVEL, nx.average_shortest_path_length),
            ("density", True, "Density", GRAPHLEVEL, nx.density),
            ("degree_assortativity_coefficient", False, \
                "Degree assortativity coefficient", GRAPHLEVEL, \
                    nx.degree_assortativity_coefficient if \
                    hasattr(nx, "degree_assortativity_coefficient") else None),
            # additional attr needed
            #("attribute_assortativity_coefficient", False, "Attribute assortativity coefficient", GRAPHLEVEL, nx.attribute_assortativity_coefficient),
            #("numeric_assortativity_coefficient", False, "Numeric assortativity coefficient", GRAPHLEVEL, nx.numeric_assortativity_coefficient),
            ("degree_pearson_correlation_coefficient", False, \
                "Degree pearson correlation coefficient", GRAPHLEVEL, \
                nx.degree_pearson_correlation_coefficient if\
                hasattr(nx, "degree_pearson_correlation_coefficient") else None),
            ("estrada_index", False, "Estrada index", GRAPHLEVEL, \
                nx.estrada_index if hasattr(nx, "estrada_index") else None),
            ("graph_clique_number", False, "Graph clique number", GRAPHLEVEL, nx.graph_clique_number),
            ("graph_number_of_cliques", False, "Graph number of cliques", GRAPHLEVEL, nx.graph_number_of_cliques),
            ("transitivity", False, "Graph transitivity", GRAPHLEVEL, nx.transitivity),
            ("average_clustering", False, "Average clustering coefficient", GRAPHLEVEL, nx.average_clustering),
            ("number_connected_components", False, "Number of connected components", GRAPHLEVEL, nx.number_connected_components),
            ("number_strongly_connected_components", False, "Number of strongly connected components", GRAPHLEVEL, nx.number_strongly_connected_components),
            ("number_weakly_connected_components", False, "Number of weakly connected components", GRAPHLEVEL, nx.number_weakly_connected_components),
            ("number_attracting_components", False, "Number of attracting components", GRAPHLEVEL, nx.number_attracting_components),
            # TODO: input parameters
            #("max_flow", False, "Maximum flow", GRAPHLEVEL, nx.max_flow),
            #("min_cut", False, "Minimum cut", GRAPHLEVEL, nx.min_cut),
            #("ford_fulkerson", False, "Maximum single-commodity flow (Ford-Fulkerson)", GRAPHLEVEL, nx.ford_fulkerson),
            #("min_cost_flow_cost", False, "min_cost_flow_cost", GRAPHLEVEL, nx.min_cost_flow_cost),
            # returns dict of dict
            #("shortest_path_length", False, "Shortest path length", GRAPHLEVEL, nx.shortest_path_length),

            ("degree", False, "Degree", NODELEVEL, nx.degree),
            ("in_degree", False, "In-degree", NODELEVEL, lambda G: G.in_degree()),
            ("out_degree", False, "Out-degree", NODELEVEL, lambda G: G.out_degree()),
            ("average_neighbor_degree", False, "Average neighbor degree", NODELEVEL, nx.average_neighbor_degree),
            ("clustering", False, "Clustering coefficient", NODELEVEL, nx.clustering),
            ("triangles", False, "Number of triangles", NODELEVEL, nx.triangles),
            ("square_clustering", False, "Squares clustering coefficient", NODELEVEL, nx.square_clustering),
            ("number_of_cliques", False, "Number of cliques", NODELEVEL, nx.number_of_cliques),
            ("degree_centrality", False, "Degree centrality", NODELEVEL, nx.degree_centrality),
            ("in_degree_centrality", False, "In-egree centrality", NODELEVEL, nx.in_degree_centrality),
            ("out_degree_centrality", False, "Out-degree centrality", NODELEVEL, nx.out_degree_centrality),
            ("closeness_centrality", False, "Closeness centrality", NODELEVEL, nx.closeness_centrality),
            ("betweenness_centrality", False, "Betweenness centrality", NODELEVEL, nx.betweenness_centrality),
            ("current_flow_closeness_centrality", False, "Information centrality", NODELEVEL, nx.current_flow_closeness_centrality),
            ("current_flow_betweenness_centrality", False, "Random-walk betweenness centrality", NODELEVEL, nx.current_flow_betweenness_centrality),
            ("approximate_current_flow_betweenness_centrality", False, \
                "Approx. random-walk betweenness centrality", NODELEVEL, \
                nx.approximate_current_flow_betweenness_centrality if \
                hasattr(nx, "approximate_current_flow_betweenness_centrality") \
                    else None),
            ("eigenvector_centrality", False, "Eigenvector centrality", NODELEVEL, nx.eigenvector_centrality),
            ("eigenvector_centrality_numpy", False, "Eigenvector centrality (NumPy)", NODELEVEL, nx.eigenvector_centrality_numpy),
            ("load_centrality", False, "Load centrality", NODELEVEL, nx.load_centrality),
            ("core_number", False, "Core number", NODELEVEL, nx.core_number),
            ("eccentricity", False, "Eccentricity", NODELEVEL, nx.eccentricity),
            ("closeness_vitality", False, "Closeness vitality", NODELEVEL, nx.closeness_vitality),
        ]
        """
        TODO: add
            average-degree_connectivity
            is_bipartite
            is_chordal
            katz_centrality
            katz_centrality_numpy
            communicability
            communicability_exp
            communicability_centrality
            communicability_centrality_exp
            communicability_betweenness_centrality
            average_node_connectivity
            is_directed_acyclic_graph
            center
            ??
        """

        self.methods = [method for method in self.methods if method[-1] is not None]

        self.auto_commit = False
        self.tab_index = 0
        self.mutex = QtCore.QMutex()

        self.graph = None
        self.items = None          # items set by Items signal
        self.items_graph = None    # items set by graph.items by Network signal
        self.items_analysis = None # items to output and merge with analysis result

        self.job_queue = []
        self.job_working = []
        self.analfeatures = []
        self.analdata = {}

        for method in self.methods:
            setattr(self, method[0], method[1])
            setattr(self, "lbl_" + method[0], "")

        self.tabs = gui.tabWidget(self.controlArea)
        self.tabs.setMinimumWidth(450)
        self.graphIndices = gui.createTabPage(self.tabs, "Graph-level indices")
        self.nodeIndices = gui.createTabPage(self.tabs, "Node-level indices")
        self.tabs.setCurrentIndex(self.tab_index)
        self.connect(self.tabs, SIGNAL("currentChanged(int)"), lambda index: setattr(self, 'tab_index', index))

        for name, default, label, type, algorithm in self.methods:
            if type == NODELEVEL:
                box = gui.widgetBox(self.nodeIndices, orientation="horizontal")
            elif type == GRAPHLEVEL:
                box = gui.widgetBox(self.graphIndices, orientation="horizontal")

            gui.checkBox(box, self, name, label=label, callback=lambda n=name: self.method_clicked(n))
            box.layout().addStretch(1)
            lbl = gui.label(box, self, "%(lbl_" + name + ")s")
            setattr(self, "tool_" + name, lbl)

        self.graphIndices.layout().addStretch(1)
        self.nodeIndices.layout().addStretch(1)

        gui.checkBox(self.controlArea, self, "auto_commit", label="Commit automatically")

        hb = gui.widgetBox(self.controlArea, None, orientation='horizontal')
        self.btnCommit = gui.button(hb, self, "Commit", callback=self.analyze, toggleButton=1)
        self.btnStopA = gui.button(hb, self, "Cancel", callback=lambda: self.stop_job(current=False))
        self.btnStopA.setEnabled(False)

        #~ self.reportButton = gui.button(hb, self, "&Report", self.reportAndFinish, debuggingEnabled=0)
        #~ self.reportButton.setAutoDefault(0)

    def set_graph(self, graph):
        if graph is None:
            return

        self.stop_job(current=False)

        self.mutex.lock()

        self.graph = graph
        self.items_graph = graph.items()
        self.items_analysis = graph.items()

        if self.items is not None:
            self.items_analysis =  self.items

        self.clear_results()
        self.clear_labels()
        # only clear computed statistics on new graph
        self.analdata.clear()

        self.mutex.unlock()

        if self.auto_commit:
            self.btnCommit.setChecked(True)
            self.analyze()


    def set_items(self, items):
        self.mutex.lock()

        if items is None and self.items_graph is not None:
            self.items_analysis = self.items_graph

        elif items is not None:
            self.items_analysis = items

        self.items = items

        self.mutex.unlock()

    def analyze(self):
        if self.graph is None or not self.btnCommit.isChecked():
            return

        if len(self.job_queue) > 0 or len(self.job_working) > 0:
            return

        self.btnStopA.setEnabled(True)
        self.clear_labels()
        qApp.processEvents()

        self.clear_results()

        for method in self.methods:
            self.add_job(method)

        if len(self.job_queue) > 0:
            self.start_job()
        else:
            self.send_data()

    def add_job(self, method):
        name, default, label, type, algorithm = method

        is_method_enabled = getattr(self, name)

        if not is_method_enabled:
            return

        #if type == NODELEVEL:
        job = WorkerThread(self, name, label, type, algorithm)
        self.connect(job, SIGNAL("terminated()"), lambda j=job: self.job_terminated(j))
        self.connect(job, SIGNAL("finished()"), lambda j=job: self.job_finished(j))
        self.job_queue.insert(0, job)
        setattr(self, "lbl_" + job.name, "   waiting")

    def start_job(self):
        max_jobs = max(1, QThread.idealThreadCount())

        self.mutex.lock()
        if len(self.job_queue) > 0 and len(self.job_working) < max_jobs:
            job = self.job_queue.pop()
            setattr(self, "lbl_" + job.name, "   started")

            # if data for this job already computed
            if job.name in self.analdata:
                if job.type == NODELEVEL:
                    self.analfeatures.append((job.name, \
                                Orange.data.ContinuousVariable(job.label)))
                    setattr(self, "lbl_" + job.name, "  finished")

                elif job.type == GRAPHLEVEL:
                    setattr(self, "lbl_" + job.name,("%.4f" % \
                            self.analdata[job.name]).rstrip('0').rstrip('.'))

                job.quit()
                self.send_data()
            else:
                self.job_working.append(job)
                job.start()
        self.mutex.unlock()

        if len(self.job_queue) > 0 and len(self.job_working) < max_jobs:
            self.start_job()

    def job_terminated(self, job):
        self.mutex.lock()
        job.is_terminated = True
        self.mutex.unlock()

    def job_finished(self, job):
        self.mutex.lock()
        if job.is_terminated:
            setattr(self, "lbl_" + job.name, "terminated")
        else:
            setattr(self, "lbl_" + job.name, "  finished")

            if job.error is not None:
                setattr(self, "lbl_" + job.name, "     error")
                tooltop = getattr(self, "tool_" + job.name)
                tooltop.setToolTip(job.error.args[0])

            elif job.result is not None:
                if job.type == NODELEVEL:
                    self.analfeatures.append((job.name, Orange.data.ContinuousVariable(job.label)))
                    self.analdata[job.name] = [job.result[node] for node in sorted(job.result.keys())]

                elif job.type == GRAPHLEVEL:
                    self.analdata[job.name] = job.result
                    setattr(self, "lbl_" + job.name, ("%.4f" % job.result).rstrip('0').rstrip('.'))

        if job in self.job_working:
            self.job_working.remove(job)

        self.send_data()
        self.mutex.unlock()

        if len(self.job_queue) > 0:
            self.start_job()

    def stop_job(self, current=True, name=None):
        self.mutex.lock()

        if name is not None:
            for i in range(len(self.job_queue) - 1, -1, -1 ):
                job = self.job_queue[i]
                if name == job.name:

                    job.is_terminated = True
                    job.quit()
                    job.wait()
                    self.job_queue.remove(job)
                    setattr(self, "lbl_" + name, "terminated")

            #~ # This was commented out because it might have hanged
            #~ for job in self.job_working:
                #~ if name == job.name:
                    #~ job.is_terminated = True
                    #~ job.terminate()
        else:
            if not current:
                while len(self.job_queue) > 0:
                    job = self.job_queue.pop()
                    job.is_terminated = True
                    job.quit()
                    job.wait()
                    setattr(self, "lbl_" + job.name, "terminated")

            #~ # This was commented out because it hanged
            #~ for job in self.job_working:
                #~ job.is_terminated = True
                #~ job.terminate()

        self.mutex.unlock()

    def send_data(self):
        if len(self.job_queue) <= 0 and len(self.job_working) <= 0:
            self.btnCommit.setChecked(False)
            self.btnStopA.setEnabled(False)

            if self.analdata is not None and len(self.analdata) > 0 and \
                                                    len(self.analfeatures) > 0:
                vars = []
                analdata = []
                for name, var in self.analfeatures:
                    analdata.append(self.analdata[name])
                    vars.append(var)

                table  = Table(Domain(vars),
                                      [list(t) for t in zip(*analdata)])
                if self.items_analysis:
                    table = Table.concatenate((table, self.items_analysis))
                self.graph.set_items(table)

            self.send("Network", self.graph)
            self.send("Items", self.graph.items())

            self.clear_results()

    def method_clicked(self, name):
        if self.auto_commit:
            self.mutex.lock()
            if len(self.job_queue) <= 0 and len(self.job_working) <= 0:
                self.btnCommit.setChecked(True)
                self.mutex.unlock()
                self.analyze()
            else:
                is_method_enabled = getattr(self, name)
                if is_method_enabled:
                    for method in self.methods:
                        if name == method[0]:
                            self.add_job(method)
                    self.mutex.unlock()
                else:
                    self.mutex.unlock()
                    self.stop_job(name=name)

    def clear_results(self):
        del self.job_queue[:]
        del self.job_working[:]
        del self.analfeatures[:]

    def clear_labels(self):
        for method in self.methods:
            setattr(self, "lbl_" + method[0], "")

    def sendReport(self):
        report = []

        for name, default, label, type, algorithm in self.methods:
            if type == GRAPHLEVEL:
                value = getattr(self, "lbl_" + name)
                value = str(value).strip().lower()
                if value != "" and value != "error"  and value != "waiting" \
                            and value != "terminated" and value != "finished":
                    report.append((label, value))

        self.reportSettings("Graph statistics", report)


if __name__ == "__main__":
    a=QApplication(sys.argv)
    ow=OWNxAnalysis()
    ow.show()
    def setNetwork(signal, data, id=None):
        if signal == 'Network':
            ow.set_graph(data)
        #if signal == 'Items':
        #    ow.set_items(data)

    from . import OWNxFile
    owFile = OWNxFile.OWNxFile()
    owFile.send = setNetwork
    owFile.show()
    owFile.selectNetFile(0)

    a.exec_()
    ow.saveSettings()
    owFile.saveSettings()



"""
=======
Network
=======

Network visualization and analysis tools for Orange.

"""

# Category description for the widget registry

NAME = "Network"

DESCRIPTION = "Network visualization and analysis tools for Orange."

BACKGROUND = "#C0FF97"

ICON = "icons/Category-Network.svg"



import sys
from PyQt4.QtGui import QApplication

appl = QApplication(sys.argv)

import Orange.data
import Orange.network
import OWNxExplorer

net = Orange.network.readwrite.read('musicians.net')
net.set_items(Orange.data.Table('musicians_items.tab'))

ow = OWNxExplorer.OWNxExplorer()
ow.set_graph(net)

ow.show()
appl.exec_()



from PyQt4.QtGui import *
from PyQt4.QtCore import *

import numpy as np
import networkx as nx

import Orange
from Orange.widgets import gui, widget, settings

import orangecontrib.network as network


def _balanced_tree(n):
    b = max(np.log(n) / 2, 2)
    h = np.log((n * (b - 1)) + 1)/np.log(b) - 1
    return nx.balanced_tree(int(b), int(h))


def _hypercube(n):
    """Hypercube's nodes are cube coordinates, which isn't so nice"""
    G = nx.hypercube_graph(int(np.log2(n) + .1))
    G = nx.relabel_nodes(G, {n: int('0b' + ''.join(str(i) for i in n), 2)
                             for n in G.node})
    return G


class GraphType:
    BALANCED_TREE = ('Balanced tree', _balanced_tree)
    BARBELL = ('Barbell', lambda n: nx.barbell_graph(int(n*.4), int(n*.3)))
    CIRCULAR_LADDER = ('Circular ladder', lambda n: nx.circular_ladder_graph(int(n/2)))
    COMPLETE = ('Complete', lambda n: nx.complete_graph(int(n)))
    COMPLETE_BIPARTITE = ('Complete bipartite',
                          lambda n: nx.complete_bipartite_graph(int(n*.6), int(n*.4)))
    CYCLE = ('Cycle', lambda n: nx.cycle_graph(int(n)))
    GRID = ('Grid', lambda n: nx.grid_graph([int(np.sqrt(n))]*2))
    HYPERCUBE = ('Hypercube', _hypercube)
    LADDER = ('Ladder', lambda n: nx.ladder_graph(int(n/2)))
    LOBSTER = ('Lobster', lambda n: nx.random_lobster(int(n / (1 + .7 + .7*.5)), .7, .5))
    LOLLIPOP = ('Lollipop', lambda n: nx.lollipop_graph(int(n/2), int(n/2)))
    PATH = ('Path', lambda n: nx.path_graph(int(n)))
    REGULAR = ('Regular', lambda n: nx.random_regular_graph(np.random.randint(10)*2, n))
    SCALEFREE = ('Scale-free', lambda n: nx.scale_free_graph(int(n)))
    SHELL = ('Shell', lambda n: nx.random_shell_graph([(int(n*.1), int(n*.1), .2),
                                                       (int(n*.3), int(n*.3), .8),
                                                       (int(n*.6), int(n*.6), .5)]))
    STAR = ('Star', lambda n: nx.star_graph(int(n - 1)))
    WAXMAN = ('Waxman', lambda n: nx.waxman_graph(int(n)))
    WHEEL = ('Wheel', lambda n: nx.wheel_graph(int(n)))

    all = (BALANCED_TREE, BARBELL, CIRCULAR_LADDER, COMPLETE, COMPLETE_BIPARTITE,
           CYCLE, GRID, HYPERCUBE, LADDER, LOBSTER, LOLLIPOP, PATH, REGULAR,
           SCALEFREE, SHELL, STAR, WAXMAN, WHEEL)


class Output:
    NETWORK = 'Generated network'


class OWNxGenerator(widget.OWWidget):
    name = "Network Generator"
    description = "Generate example graphs."
    icon = "icons/NetworkGenerator.svg"
    priority = 6420

    outputs = [(Output.NETWORK, network.Graph),]

    graph_type = settings.Setting(0)
    n_nodes = settings.Setting(50)
    auto_commit = settings.Setting(True)

    want_main_area = False
    resizing_enabled = False

    def __init__(self):
        super().__init__()
        gui.comboBox(self.controlArea, self, 'graph_type',
                     label='Generate graph:',
                     items=GraphType.all,
                     orientation='horizontal',
                     callback=self.generate)
        gui.spin(self.controlArea, self, 'n_nodes',
                 10, 500, 10,
                 label='Approx. number of nodes:',
                 orientation='horizontal',
                 callbackOnReturn=True,
                 callback=self.generate)
        gui.auto_commit(self.controlArea, self, 'auto_commit',
                        label='Generate graph',
                        checkbox_label='Auto-generate')
        self.commit()

    def generate(self):
        return self.commit()

    def commit(self):
        _, func = GraphType.all[self.graph_type]
        graph = network.readwrite._wrap(func(self.n_nodes))
        self.send(Output.NETWORK, graph)


if __name__ == "__main__":
    a = QApplication([])
    ow = OWNxGenerator()
    ow.show()
    a.exec_()
    ow.saveSettings()



from Orange.widgets import gui, widget, settings
from orangecontrib.network import Graph, community as cd


class OWNxClustering(widget.OWWidget):
    name = 'Network Clustering'
    description = 'Orange widget for community detection in networks.'
    icon = "icons/NetworkClustering.svg"
    priority = 6430

    inputs = [("Network", Graph, "setNetwork", widget.Default)]
    outputs = [("Network", Graph),
               ("Community Detection", cd.CommunityDetection)]

    resizing_enabled = False

    method = settings.Setting(0)
    iterations = settings.Setting(1000)
    hop_attenuation = settings.Setting(0.1)
    autoApply = settings.Setting(False)

    def __init__(self):
        super().__init__()

        self.net = None

        commit = lambda: self.commit()
        gui.spin(self.controlArea, self, "iterations", 1,
                   100000, 1, label="Max. iterations:",
                   callback=commit)
        ribg = gui.radioButtonsInBox(
            self.controlArea, self, "method",
            btnLabels=["Label propagation clustering (Raghavan et al., 2007)",
                    "Label propagation clustering (Leung et al., 2009)"],
            box="Clustering method", callback=commit)

        gui.doubleSpin(gui.indentedBox(ribg), self, "hop_attenuation",
                         0, 1, 0.01, label="Hop attenuation (delta): ")

        self.info = gui.widgetLabel(self.controlArea, ' ')

        gui.auto_commit(self.controlArea, self, "autoApply", 'Commit',
                        checkbox_label='Auto-commit')
        commit()

    def setNetwork(self, net):
        self.net = net
        self.commit()

    def commit(self):
        self.info.setText(' ')

        if self.method == 0:
            alg = cd.label_propagation
            kwargs = {'results2items': 1,
                      'iterations': self.iterations}

        elif self.method == 1:
            alg = cd.label_propagation_hop_attenuation
            kwargs = {'results2items': 1,
                      'iterations': self.iterations,
                      'delta': self.hop_attenuation}

        self.send("Community Detection", cd.CommunityDetection(alg, **kwargs))

        if self.net is None:
            self.send("Network", None)
            return

        labels = alg(self.net, **kwargs)

        self.info.setText('%d clusters found' % len(set(labels.values())))
        self.send("Network", self.net)


if __name__ == "__main__":
    from PyQt4.QtGui import *
    a = QApplication([])
    ow = OWNxClustering()
    ow.show()

    def setNetwork(signal, data, id=None):
        if signal == 'Network':
            ow.setNetwork(data)

    import OWNxFile
    from os.path import join, dirname
    owFile = OWNxFile.OWNxFile()
    owFile.send = setNetwork
    owFile.openFile(join(dirname(dirname(__file__)), 'networks', 'leu_by_genesets.net'))

    a.exec_()
    ow.saveSettings()
    owFile.saveSettings()



#
# OWHist.py
#
# the base for network histograms

import math
from itertools import chain
import numpy as np

from PyQt4.QtGui import *
from PyQt4.QtCore import *

import Orange
from Orange.widgets import gui, widget, settings
import orangecontrib.network as network

import pyqtgraph as pg


class NodeSelection:
    ALL_NODES,   \
    COMPONENTS,  \
    LARGEST_COMP = range(3)

class EdgeWeights:
    PROPORTIONAL, \
    INVERSE = range(2)


class OWNxFromDistances(widget.OWWidget):
    name = "Network from Distances"
    description = ('Constructs Graph object by connecting nodes from '
                   'data table where distance between them is between '
                   'given threshold.')
    icon = "icons/NetworkFromDistances.svg"
    priority = 6440

    inputs = [("Distances", Orange.misc.DistMatrix, "setMatrix")]
    outputs = [("Network", network.Graph),
               ("Data", Orange.data.Table),
               ("Distances", Orange.misc.DistMatrix)]

    resizing_enabled = False

    # TODO: make settings input-dependent
    percentil = settings.Setting(1)
    include_knn = settings.Setting(False)
    kNN = settings.Setting(2)
    node_selection = settings.Setting(0)
    edge_weights = settings.Setting(0)
    excludeLimit = settings.Setting(2)

    def __init__(self):
        super().__init__()

        self.spinUpperThreshold = 0

        self.matrix = None
        self.graph = None
        self.graph_matrix = None

        self.histogram = Histogram(self)
        self.mainArea.layout().addWidget(self.histogram)
        self.mainArea.setMinimumWidth(500)
        self.mainArea.setMinimumHeight(300)
        self.addHistogramControls()

        # info
        boxInfo = gui.widgetBox(self.controlArea, box="Info")
        self.infoa = gui.widgetLabel(boxInfo, "No data loaded.")
        self.infob = gui.widgetLabel(boxInfo, '')
        self.infoc = gui.widgetLabel(boxInfo, '')

        gui.rubber(self.controlArea)

        self.resize(700, 100)

    def addHistogramControls(self):
        boxGeneral = gui.widgetBox(self.controlArea, box="Edges")
        ribg = gui.widgetBox(boxGeneral, None, orientation="horizontal", addSpace=False)
        self.spin_high = gui.doubleSpin(boxGeneral, self, 'spinUpperThreshold',
                                        0, float('inf'), 0.001, decimals=3,
                                        label='Distance threshold',
                                        callback=self.changeUpperSpin,
                                        keyboardTracking=False,
                                        controlWidth=60)
        self.histogram.region.sigRegionChangeFinished.connect(self.spinboxFromHistogramRegion)

        ribg = gui.widgetBox(boxGeneral, None, orientation="horizontal", addSpace=False)

        gui.doubleSpin(boxGeneral, self, "percentil", 0, 100, 0.1,
                      label="Percentile", orientation='horizontal',
                      callback=self.setPercentil,
                      callbackOnReturn=1, controlWidth=60)

        hbox = gui.widgetBox(boxGeneral, orientation='horizontal')
        knn_cb = gui.checkBox(hbox, self, 'include_knn',
                              label='Include also closest neighbors',
                              callback=self.generateGraph)
        knn = gui.spin(hbox, self, "kNN", 1, 1000, 1,
                       orientation='horizontal',
                       callback=self.generateGraph, callbackOnReturn=1, controlWidth=60)
        knn_cb.disables = [knn]
        knn_cb.makeConsistent()

        ribg.layout().addStretch(1)
        # Options
        ribg = gui.radioButtonsInBox(self.controlArea, self, "node_selection",
                                     box="Node selection",
                                     callback=self.generateGraph)
        gui.appendRadioButton(ribg, "Keep all nodes")
        hb = gui.widgetBox(ribg, None, orientation="horizontal", addSpace=False)
        gui.appendRadioButton(ribg, "Components with at least nodes", insertInto=hb)
        gui.spin(hb, self, "excludeLimit", 2, 100, 1,
                 callback=(lambda h=True: self.generateGraph(h)), controlWidth=60)
        gui.appendRadioButton(ribg, "Largest connected component")
        self.attribute = None

        ribg = gui.radioButtonsInBox(self.controlArea, self, "edge_weights",
                                     box="Edge weights",
                                     callback=self.generateGraph)
        hb = gui.widgetBox(ribg, None, orientation="horizontal", addSpace=False)
        gui.appendRadioButton(ribg, "Proportional to distance", insertInto=hb)
        gui.appendRadioButton(ribg, "Inverted distance", insertInto=hb)

    def setPercentil(self):
        if self.matrix is None or self.percentil <= 0:
            return
        # flatten matrix, sort values and remove identities (self.matrix[i][i])
        ind = int(len(self.matrix_values) * self.percentil / 100)
        self.spinUpperThreshold = self.matrix_values[ind]
        self.generateGraph()

    def setMatrix(self, data):
        self.matrix = data
        if data is None:
            self.histogram.setValues([])
            self.generateGraph()
            return

        if self.matrix.row_items is None:
            self.matrix.row_items = list(range(self.matrix.shape[0]))

        # draw histogram
        self.matrix_values = values = sorted(self.matrix.flat)
        self.histogram.setValues(values)

        # Magnitude of the spinbox's step is data-dependent
        low, upp = values[0], values[-1]
        step = (upp - low) / 20
        self.spin_high.setSingleStep(step)

        self.spinUpperThreshold = low - (0.03 * (upp - low))

        self.setPercentil()
        self.generateGraph()

    def changeUpperSpin(self):
        if self.matrix is None: return
        self.spinUpperThreshold = np.clip(self.spinUpperThreshold, *self.histogram.boundary())
        self.percentil = 100 * np.searchsorted(self.matrix_values, self.spinUpperThreshold) / len(self.matrix_values)
        self.generateGraph()

    def spinboxFromHistogramRegion(self):
        _, self.spinUpperThreshold = self.histogram.getRegion()
        self.changeUpperSpin()

    def generateGraph(self, N_changed=False):
        self.error()
        matrix = None
        self.warning('')

        if N_changed:
            self.node_selection = NodeSelection.COMPONENTS

        if self.matrix is None:
            if hasattr(self, "infoa"):
                self.infoa.setText("No data loaded.")
            if hasattr(self, "infob"):
                self.infob.setText("")
            if hasattr(self, "infoc"):
                self.infoc.setText("")
            self.pconnected = 0
            self.nedges = 0
            self.graph = None
            self.sendSignals()
            return

        nEdgesEstimate = 2 * sum(y for x, y in zip(self.histogram.xData, self.histogram.yData)
                                 if x <= self.spinUpperThreshold)

        if nEdgesEstimate > 200000:
            self.graph = None
            nedges = 0
            n = 0
            self.error('Estimated number of edges is too high (%d).' % nEdgesEstimate)
        else:
            graph = network.Graph()
            graph.add_nodes_from(range(self.matrix.shape[0]))
            matrix = self.matrix

            if matrix is not None and matrix.row_items is not None:
                if isinstance(self.matrix.row_items, Orange.data.Table):
                    graph.set_items(self.matrix.row_items)
                else:
                    data = [[str(x)] for x in self.matrix.row_items]
                    items = Orange.data.Table(Orange.data.Domain([], metas=[Orange.data.StringVariable('label')]), data)
                    graph.set_items(items)

            # set the threshold
            # set edges where distance is lower than threshold
            self.warning(0)
            if self.kNN >= self.matrix.shape[0]:
                self.warning(0, "kNN larger then supplied distance matrix dimension. Using k = %i" % (self.matrix.shape[0] - 1))

            def edges_from_distance_matrix(matrix, upper, knn):
                rows, cols = matrix.shape
                for i in range(rows):
                    for j in range(i + 1, cols):
                        if matrix[i, j] <= upper:
                            yield i, j, matrix[i, j]
                    if not knn: continue
                    for j in np.argsort(matrix[i])[:knn]:
                        yield i, j, matrix[i, j]

            edge_list = edges_from_distance_matrix(
                self.matrix, self.spinUpperThreshold,
                min(self.kNN, self.matrix.shape[0] - 1) if self.include_knn else 0)
            if self.edge_weights == EdgeWeights.INVERSE:
                edge_list = list(edge_list)
                max_weight = max(d for u, v, d in edge_list)
                graph.add_edges_from((u, v, {'weight': max_weight - d})
                                     for u, v, d in edge_list)
            else:
                graph.add_edges_from((u, v, {'weight': d})
                                     for u, v, d in edge_list)
            matrix = None
            self.graph = None
            component = []
            # exclude unconnected
            if self.node_selection == NodeSelection.COMPONENTS:
                component = list(chain.from_iterable(x for x in network.nx.connected_components(graph)
                                                     if len(x) >= self.excludeLimit))
            # largest connected component only
            elif self.node_selection == NodeSelection.LARGEST_COMP:
                component = max(network.nx.connected_components(graph), key=len)
            else:
                self.graph = graph
            if len(component) > 1:
                if len(component) == graph.number_of_nodes():
                    self.graph = graph
                    matrix = self.matrix
                else:
                    self.graph = graph.subgraph(component)
                    matrix = self.matrix.submatrix(sorted(component))

        if matrix is not None:
            matrix.row_items = self.graph.items()
        self.graph_matrix = matrix

        if self.graph is None:
            self.pconnected = 0
            self.nedges = 0
        else:
            self.pconnected = self.graph.number_of_nodes()
            self.nedges = self.graph.number_of_edges()
        if hasattr(self, "infoa"):
            self.infoa.setText("Data items on input: %d" % self.matrix.shape[0])
        if hasattr(self, "infob"):
            self.infob.setText("Network nodes: %d (%3.1f%%)" % (self.pconnected,
                self.pconnected / float(self.matrix.shape[0]) * 100))
        if hasattr(self, "infoc"):
            self.infoc.setText("Network edges: %d (%.2f edges/node)" % (
                self.nedges, self.nedges / float(self.pconnected)
                if self.pconnected else 0))

        self.warning(303)
        if self.pconnected > 1000 or self.nedges > 2000:
            self.warning(303, 'Large number of nodes/edges; performance will be hindered.')

        self.sendSignals()
        self.histogram.setRegion(0, self.spinUpperThreshold)

    def sendReport(self):
        self.reportSettings("Settings",
                            [("Edge thresholds", "%.5f - %.5f" % \
                              (0, \
                               self.spinUpperThreshold)),
                             ("Selected vertices", ["All", \
                                "Without isolated vertices",
                                "Largest component",
                                "Connected with vertex"][self.node_selection]),
                             ("Weight", ["Distance", "1 - Distance"][self.edge_weights])])
        self.reportSection("Histogram")
        self.reportImage(self.histogram.saveToFileDirect, QSize(400,300))
        self.reportSettings("Output graph",
                            [("Vertices", self.matrix.dim),
                             ("Edges", self.nedges),
                             ("Connected vertices", "%i (%.1f%%)" % \
                              (self.pconnected, self.pconnected / \
                               max(1, float(self.matrix.dim))*100))])

    def sendSignals(self):
        self.send("Network", self.graph)
        self.send("Distances", self.graph_matrix)
        if self.graph == None:
            self.send("Data", None)
        else:
            self.send("Data", self.graph.items())


pg_InfiniteLine = pg.InfiniteLine

class InfiniteLine(pg_InfiniteLine):
    def paint(self, p, *args):
        # From orange3-bioinformatics:OWFeatureSelection.py, thanks to @ales-erjavec
        brect = self.boundingRect()
        c = brect.center()
        line = QLineF(brect.left(), c.y(), brect.right(), c.y())
        t = p.transform()
        line = t.map(line)
        p.save()
        p.resetTransform()
        p.setPen(self.currentPen)
        p.drawLine(line)
        p.restore()

# Patched so that the Histogram's LinearRegionItem works on MacOS
pg.InfiniteLine = InfiniteLine
pg.graphicsItems.LinearRegionItem.InfiniteLine = InfiniteLine


class Histogram(pg.PlotWidget):
    def __init__(self, parent, **kwargs):
        super().__init__(parent, setAspectLocked=True, **kwargs)
        self.curve = self.plot([0, 1], [0], pen=pg.mkPen('b', width=2), stepMode=True)
        self.region = pg.LinearRegionItem([0, 0], brush=pg.mkBrush('#02f1'), movable=True)
        self.region.sigRegionChanged.connect(self._update_region)
        # Selected region is only open-ended on the the upper side
        self.region.hoverEvent = self.region.mouseDragEvent = lambda *args: None
        self.region.lines[0].setVisible(False)
        self.addItem(self.region)
        self.fillCurve = self.plotItem.plot([0, 1], [0],
            fillLevel=0, pen=pg.mkPen('b', width=2), brush='#02f3', stepMode=True)
        self.plotItem.vb.setMouseEnabled(x=False, y=False)

    def _update_region(self, region):
        rlow, rhigh = self.getRegion()
        low = max(0, np.searchsorted(self.xData, rlow, side='right') - 1)
        high = np.searchsorted(self.xData, rhigh, side='right')
        if high - low > 0:
            xData = self.xData[low:high + 1].copy()
            xData[0] = rlow  # set visible boundaries to match region lines
            xData[-1] = rhigh
            self.fillCurve.setData(xData, self.yData[low:high])

    def setBoundary(self, low, high):
        self.region.setBounds((low, high))

    def boundary(self):
        return self.xData[[0, -1]]

    def setRegion(self, low, high):
        low, high = np.clip([low, high], *self.boundary())
        self.region.setRegion((low, high))

    def getRegion(self):
        return self.region.getRegion()

    def setValues(self, values):
        self.fillCurve.setData([0,1], [0])
        if not len(values):
            self.curve.setData([0, 1], [0])
            self.setBoundary(0, 0)
            return
        nbins = int(min(np.sqrt(len(values)), 100))
        freq, edges = np.histogram(values, bins=nbins)
        self.curve.setData(edges, freq)
        self.setBoundary(edges[0], edges[-1])
        self.autoRange()

    @property
    def xData(self):
        return self.curve.xData

    @property
    def yData(self):
        return self.curve.yData


if __name__ == "__main__":
    appl = QApplication([])
    ow = OWNxFromDistances()
    ow.show()
    appl.exec_()



from threading import Thread

import numpy as np
import networkx as nx

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import QLineF, QPointF, QRectF, Qt
from PyQt4.QtGui import qApp, QBrush, QPen, QPolygonF, QStyle, QColor

from orangecontrib.network._fr_layout import fruchterman_reingold_layout

# Expose OpenGL rendering for large graphs, if available
HAVE_OPENGL = True
try: from PyQt4 import QtOpenGL
except: HAVE_OPENGL = False

FR_ITERATIONS = 60

IS_LARGE_GRAPH = lambda G: G.number_of_nodes() + G.number_of_edges() > 4000
IS_VERY_LARGE_GRAPH = lambda G: G.number_of_nodes() + G.number_of_edges() > 10000


class QGraphicsEdge(QtGui.QGraphicsLineItem):
    def __init__(self, source, dest, view=None):
        super().__init__()
        self.setAcceptedMouseButtons(Qt.NoButton)
        self.setFlags(self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity)
        self.setZValue(1)
        self.setPen(QPen(Qt.gray, .7))

        source.addEdge(self)
        dest.addEdge(self)
        self.source = source
        self.dest = dest
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('test', self)
        label.setVisible(False)
        label.setBrush(Qt.gray)
        label.setZValue(2)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        view.scene().addItem(label)

        self.adjust()

    def adjust(self):
        line = QLineF(self.mapFromItem(self.source, 0, 0),
                      self.mapFromItem(self.dest, 0, 0))
        self.label.setPos(line.pointAt(.5))
        self.setLine(self.__transform().map(line))


class Edge(QGraphicsEdge):
    def __init__(self, source, dest, view=None):
        super().__init__(source, dest, view)
        self.setSize(.7)

    def setSize(self, size):
        self.setPen(QPen(self.pen().color(), size))

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color):
        self.setPen(QPen(QColor(color or Qt.gray), self.pen().width()))


class QGraphicsNode(QtGui.QGraphicsEllipseItem):
    """This class is the bare minimum to sustain a connected graph"""
    def __init__(self, rect=QRectF(-5, -5, 10, 10), view=None):
        super().__init__(rect)
        self.setCacheMode(self.DeviceCoordinateCache)
        self.setAcceptHoverEvents(True)
        self.setFlags(self.ItemIsMovable |
                      self.ItemIsSelectable |
                      self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity |
                      self.ItemSendsGeometryChanges)
        self.setZValue(4)

        self.edges = []
        self._radius = rect.width() / 2
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('test', self)
        label.setVisible(False)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        label.setZValue(3)
        view.scene().addItem(label)

    def setPos(self, x, y):
        self.adjust()
        super().setPos(x, y)

    def adjust(self):
        # Adjust label position
        d = 1 / self.__transform().m11() * self._radius
        self.label.setPos(self.pos().x() + d, self.pos().y() + d)

    def addEdge(self, edge):
        self.edges.append(edge)

    def itemChange(self, change, value):
        if change == self.ItemPositionHasChanged:
            self.adjust()
            for edge in self.edges:
                edge.adjust()
        return super().itemChange(change, value)


class Node(QGraphicsNode):
    """
    This class provides an interface for all the bells & whistles of the
    Network Explorer.
    """

    BRUSH_DEFAULT = QBrush(QColor('#669'))

    class Pen:
        DEFAULT = QPen(Qt.black, 0)
        SELECTED = QPen(QColor('#dd0000'), 3)
        HIGHLIGHTED = QPen(QColor('#ffaa22'), 3)

    _TOOLTIP = lambda: ''

    def __init__(self, id, view):
        super().__init__(view=view)
        self.id = id
        self.setBrush(Node.BRUSH_DEFAULT)
        self.setPen(Node.Pen.DEFAULT)

        self._is_highlighted = False
        self._tooltip = Node._TOOLTIP

    def setSize(self, size):
        self._radius = radius = size/2
        self.setRect(-radius, -radius, size, size)

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color):
        self.setBrush(QBrush(QColor(color)) if color else Node.BRUSH_DEFAULT)

    def isHighlighted(self):
        return self._is_highlighted
    def setHighlighted(self, highlight):
        self._is_highlighted = highlight
        if not self.isSelected():
            self.itemChange(self.ItemSelectedChange, False)

    def itemChange(self, change, value):
        if change == self.ItemSelectedChange:
            self.setPen(Node.Pen.SELECTED if value else
                        Node.Pen.HIGHLIGHTED if self._is_highlighted else
                        Node.Pen.DEFAULT)
        return super().itemChange(change, value)

    def paint(self, painter, option, widget):
        option.state &= ~QStyle.State_Selected  # We use a custom selection pen
        super().paint(painter, option, widget)

    def setTooltip(self, callback):
        assert not callback or callable(callback)
        self._tooltip = callback or Node._TOOLTIP
    def hoverEnterEvent(self, event):
        self.setToolTip(self._tooltip())
    def hoverLeaveEvent(self, event):
        self.setToolTip('');


class GraphView(QtGui.QGraphicsView):

    positionsChanged = QtCore.pyqtSignal(np.ndarray)
    # Emitted when nodes' selected or highlighted state changes
    selectionChanged = QtCore.pyqtSignal()
    # Emitted when the relayout() animation finishes
    animationFinished = QtCore.pyqtSignal()


    def __init__(self, parent=None):
        super().__init__(parent)
        self.nodes = []
        self.edges = []
        self._selection = []
        self._clicked_node = None
        self.is_animating = False

        scene = QtGui.QGraphicsScene(self)
        scene.setItemIndexMethod(scene.BspTreeIndex)
        scene.setBspTreeDepth(2)
        self.setScene(scene)
        self.setText('')

        self.setCacheMode(self.CacheBackground)
        self.setViewportUpdateMode(self.FullViewportUpdate)  # BoundingRectViewportUpdate doesn't work on Mac

        self.setTransformationAnchor(self.AnchorUnderMouse)
        self.setResizeAnchor(self.AnchorViewCenter)

        self.setDragMode(self.RubberBandDrag)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.positionsChanged.connect(self._update_positions, type=Qt.BlockingQueuedConnection)
        self.animationFinished.connect(lambda: setattr(self, 'is_animating', False))

    def mousePressEvent(self, event):
        self._selection = []
        if event.button() == Qt.LeftButton:
            if self.is_animating:
                self.is_animating = False
                return
            # Save the current selection and restore it on mouse{Move,Release}
            self._clicked_node = self.itemAt(event.pos())
            if event.modifiers() & Qt.ShiftModifier:
                self._selection = self.scene().selectedItems()
        # On right mouse button, switch to pan mode
        elif event.button() == Qt.RightButton:
            self.setDragMode(self.ScrollHandDrag)
            # Forge left mouse button event
            event = QtGui.QMouseEvent(event.type(),
                                      event.pos(),
                                      event.globalPos(),
                                      Qt.LeftButton,
                                      event.buttons(),
                                      event.modifiers())
        super().mousePressEvent(event)
        # Reselect the selection that had just been discarded
        for node in self._selection: node.setSelected(True)
    def mouseMoveEvent(self, event):
        super().mouseMoveEvent(event)
        if not self._clicked_node:
            for node in self._selection: node.setSelected(True)
    def mouseReleaseEvent(self, event):
        super().mouseReleaseEvent(event)
        if self.dragMode() == self.RubberBandDrag:
            for node in self._selection: node.setSelected(True)
            self.selectionChanged.emit()
        if self._clicked_node:
            self.selectionChanged.emit()
        # The following line is required (QTBUG-48443)
        self.setDragMode(self.NoDrag)
        # Restore default drag mode
        self.setDragMode(self.RubberBandDrag)

    def setText(self, text):
        text = self._text = QtGui.QStaticText(text or '')
        text.setPerformanceHint(text.AggressiveCaching)
        option = QtGui.QTextOption()
        option.setWrapMode(QtGui.QTextOption.NoWrap)
        text.setTextOption(option)
        scene = self.scene()
        scene.invalidate(layers=scene.BackgroundLayer)
    def drawForeground(self, painter, rect):
        painter.resetTransform()
        painter.drawStaticText(10, 10, self._text)
        super().drawForeground(painter, rect)
    def scrollContentsBy(self, dx, dy):
        scene = self.scene()
        scene.invalidate(layers=scene.BackgroundLayer)
        super().scrollContentsBy(dx, dy)

    def _setState(self, nodes, extend, state_setter):
        nodes = set(nodes)
        if extend:
            for node in self.nodes:
                if node.id in nodes:
                    getattr(node, state_setter)(True)
        else:
            for node in self.nodes:
                getattr(node, state_setter)(node.id in nodes)
        self.selectionChanged.emit()
    def getSelected(self):
        return [node.id for node in self.scene().selectedItems()]
    def setSelected(self, nodes, extend=False):
        self._setState(nodes, extend, 'setSelected')
    def getHighlighted(self):
        return [node.id for node in self.nodes
                if node.isHighlighted() and not node.isSelected()]
    def setHighlighted(self, nodes):
        self._setState(nodes, False, 'setHighlighted')

    def clear(self):
        self.scene().clear()
        self.scene().setSceneRect(QRectF())
        self.nodes.clear()
        self.edges.clear()

    def set_graph(self, graph, relayout=True):
        assert not graph or isinstance(graph, nx.Graph)
        self.graph = graph
        if not graph:
            self.clear()
            return
        large_graph = IS_LARGE_GRAPH(graph)
        very_large_graph = IS_VERY_LARGE_GRAPH(graph)
        self.setViewport(QtOpenGL.QGLWidget()
                         # FIXME: Try reenable the following test after Qt5 port
                         if large_graph and HAVE_OPENGL else
                         QtGui.QWidget())
        self.setRenderHints(QtGui.QPainter.RenderHint() if very_large_graph else
                            (QtGui.QPainter.Antialiasing |
                             QtGui.QPainter.TextAntialiasing))
        self.clear()
        nodes = {}
        for v in sorted(graph.nodes()):
            node = Node(v, view=self)
            self.addNode(node)
            nodes[v] = node
        for u, v in graph.edges():
            self.addEdge(Edge(nodes[u], nodes[v], view=self))
        self.selectionChanged.emit()
        if relayout: self.relayout()

    def addNode(self, node):
        assert isinstance(node, Node)
        self.nodes.append(node)
        self.scene().addItem(node)
    def addEdge(self, edge):
        assert isinstance(edge, Edge)
        self.edges.append(edge)
        self.scene().addItem(edge)

    def wheelEvent(self, event):
        if event.orientation() != Qt.Vertical: return
        self.scaleView(2**(event.delta() / 240))
    def scaleView(self, factor):
        magnitude = self.matrix().scale(factor, factor).mapRect(QRectF(0, 0, 1, 1)).width()
        if 0.2 < magnitude < 30:
            self.scale(factor, factor)
        # Reposition nodes' labela and edges, both of which are node-dependend
        # (and nodes just "moved")
        for node in self.nodes: node.adjust()
        for edge in self.edges: edge.adjust()

    @property
    def is_animating(self):
        return self._is_animating
    @is_animating.setter
    def is_animating(self, value):
        self.setCursor(Qt.ForbiddenCursor if value else Qt.ArrowCursor)
        self._is_animating = value
    def relayout(self, randomize=True, weight=None):
        if self.is_animating: return
        self.is_animating = True
        if weight is None: weight = 'weight'
        pos, graphview = None, self
        if not randomize:
            pos = [[pos.x(), pos.y()]
                   for pos in (node.pos()
                               for node in self.nodes)]

        class AnimationThread(Thread):
            def __init__(self, iterations, callback):
                super().__init__()
                self.iterations = iterations
                self.callback = callback
            def run(self):
                newpos = fruchterman_reingold_layout(graphview.graph,
                                                     pos=pos,
                                                     weight=weight,
                                                     iterations=self.iterations,
                                                     callback=self.callback)
                if not self.callback:  # update once at the end
                    graphview.update_positions(newpos)
                graphview.animationFinished.emit()

        iterations, callback = FR_ITERATIONS, self.update_positions
        if IS_VERY_LARGE_GRAPH(self.graph):
            # Don't animate very large graphs
            iterations, callback = 5, None
        AnimationThread(iterations, callback).start()

    def update_positions(self, positions):
        self.positionsChanged.emit(positions)
        return self._is_animating
    def _update_positions(self, positions):
        for node, pos in zip(self.nodes, positions*300):
            node.setPos(*pos)
        qApp.processEvents()


if __name__ == '__main__':
    import sys
    app = QtGui.QApplication(sys.argv)
    widget = GraphView()
    widget.show()
    import networkx as nx
    G = nx.scale_free_graph(int(sys.argv[1]) if len(sys.argv) > 1 else 100, seed=0)

    widget.set_graph(G)
    print('nodes', len(widget.nodes), 'edges', len(widget.edges))

    for node in widget.nodes[:10]:
        node.setHighlighted(True)

    sys.exit(app.exec_())




import numpy as np

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import QLineF, QPointF, QRectF, Qt
from PyQt4.QtGui import qApp, QBrush, QPen, QPolygonF, QStyle, QColor

from orangecontrib.network._fr_layout import fruchterman_reingold_layout

import pyqtgraph as pg

# Expose OpenGL rendering for large graphs, if available
HAVE_OPENGL = True
try: from PyQt4 import QtOpenGL
except: HAVE_OPENGL = False


class QGraphicsEdge(QtGui.QGraphicsLineItem):
    def __init__(self, source, dest, view=None):
        super().__init__()
        self.setAcceptedMouseButtons(Qt.NoButton)
        self.setFlags(self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity)
        self.setZValue(1)
        self.setPen(QPen(Qt.darkGray, .5))

        source.addEdge(self)
        dest.addEdge(self)
        self.source = source
        self.dest = dest
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('', self)
        label.setVisible(False)
        label.setBrush(Qt.gray)
        label.setZValue(2)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        view.scene().addItem(label)

        self.adjust()

    def adjust(self):
        line = QLineF(self.mapFromItem(self.source, 0, 0),
                      self.mapFromItem(self.dest, 0, 0))
        self.label.setPos(line.pointAt(.5))
        self.setLine(self.__transform().map(line))


class Edge(QGraphicsEdge):
    def __init__(self, source, dest, view=None):
        super().__init__(source, dest, view)
        self.setSize(.5)

    def setSize(self, size):
        self.setPen(QPen(self.pen().color(), size))

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color):
        self.setPen(QPen(QColor(color), self.pen().width()))


class QGraphicsNode(QtGui.QGraphicsEllipseItem):
    """This class is the bare minimum to sustain a connected graph"""
    def __init__(self, rect=QRectF(-5, -5, 10, 10), view=None):
        super().__init__(rect)
        self.setCacheMode(self.DeviceCoordinateCache)
        self.setAcceptHoverEvents(True)
        self.setFlags(self.ItemIsMovable |
                      self.ItemIsSelectable |
                      self.ItemIgnoresTransformations |
                      self.ItemIgnoresParentOpacity |
                      self.ItemSendsGeometryChanges)
        self.setZValue(4)

        self.edges = []
        self._radius = rect.width() / 2
        self.__transform = view.transform
        # Add text labels
        label = self.label = QtGui.QGraphicsSimpleTextItem('', self)
        label.setVisible(False)
        label.setFlags(self.ItemIgnoresParentOpacity |
                       self.ItemIgnoresTransformations)
        label.setZValue(3)
        view.scene().addItem(label)

    def setPos(self, x, y):
        self.adjust()
        super().setPos(x, y)

    def adjust(self):
        # Adjust label position
        d = 1 / self.__transform().m11() * self._radius
        self.label.setPos(self.pos().x() + d, self.pos().y() + d)

    def addEdge(self, edge):
        self.edges.append(edge)

    def itemChange(self, change, value):
        if change == self.ItemPositionHasChanged:
            self.adjust()
            for edge in self.edges:
                edge.adjust()
        return super().itemChange(change, value)


class Node(QGraphicsNode):
    """
    This class provides an interface for all the bells & whistles of the
    Network Explorer.
    """

    BRUSH_DEFAULT = QBrush(QColor('#669'))

    class Pen:
        DEFAULT = QPen(Qt.black, 0)
        SELECTED = QPen(QColor('#ee0000'), 3)
        HIGHLIGHTED = QPen(QColor('#ff7700'), 3)

    _tooltip = lambda _: ''

    def __init__(self, id, view):
        super().__init__(view=view)
        self.id = id
        self.setBrush(Node.BRUSH_DEFAULT)
        self.setPen(Node.Pen.DEFAULT)

        self._is_highlighted = False

    def setSize(self, size):
        self._radius = radius = size/2
        self.setRect(-radius, -radius, size, size)

    def setText(self, text):
        if text: self.label.setText(text)
        self.label.setVisible(bool(text))

    def setColor(self, color=None):
        self.setBrush(QBrush(QColor(color)) if color else Node.BRUSH_DEFAULT)

    def isHighlighted(self):
        return self._is_highlighted
    def setHighlighted(self, highlight):
        self._is_highlighted = highlight
        if not self.isSelected():
            self.itemChange(self.ItemSelectedChange, False)

    def itemChange(self, change, value):
        if change == self.ItemSelectedChange:
            self.setPen(Node.Pen.SELECTED if value else
                        Node.Pen.HIGHLIGHTED if self._is_highlighted else
                        Node.Pen.DEFAULT)
        return super().itemChange(change, value)

    def paint(self, painter, option, widget):
        option.state &= ~QStyle.State_Selected  # We use a custom selection pen
        super().paint(painter, option, widget)

    def setTooltip(self, callback):
        self._tooltip = callback
    def hoverEnterEvent(self, event):
        self.setToolTip(self._tooltip())
    def hoverLeaveEvent(self, event):
        self.setToolTip('');

    #~ def mousePressEvent(self, event):
        #~ self.update()
        #~ super().mousePressEvent(event)
#~
    #~ def mouseReleaseEvent(self, event):
        #~ self.update()
        #~ super().mouseReleaseEvent(event)


class GraphView(pg.PlotWidget):

    positionsChanged = QtCore.pyqtSignal(np.ndarray)
    # Emitted when nodes' selected or highlighted state changes
    selectionChanged = QtCore.pyqtSignal()

    def __init__(self):
        #~ super().__init__()
        pg.GraphicsView.__init__(self, background='w')
        self.nodes = []
        self.edges = []
        self._selection = []
        self._clicked_node = None

        scene = pg.GraphicsScene(self)
        scene.setItemIndexMethod(scene.NoIndex)
        #~ scene.setSceneRect(-400, -400, 800, 800)
        self.sceneObj = scene
        scene.setClickRadius(1)  ## ??? fuck this shit
        self.setScene(scene)
        self.setCacheMode(self.CacheBackground)
        self.setViewportUpdateMode(self.BoundingRectViewportUpdate)

        self.setRenderHint(QtGui.QPainter.Antialiasing)

        self.setTransformationAnchor(self.AnchorUnderMouse)
        self.setResizeAnchor(self.AnchorViewCenter)

        #~ self.setDragMode(self.ScrollHandDrag)
        self.setDragMode(self.RubberBandDrag)

        #~ self.setMinimumSize(400, 400)
        #~ self.positionsChanged.connect(self._update_positions, type=Qt.BlockingQueuedConnection)
        self.positionsChanged.connect(self._update_positions)

    def mousePressEvent(self, event):
        if self._is_animating:
            self._is_animating = False
            self.setCursor(Qt.ArrowCursor)
        # Save the current selection and restore it on mouse{Move,Release}
        self._selection = []
        self._clicked_node = self.itemAt(event.pos())
        if event.modifiers() & (Qt.ControlModifier | Qt.ShiftModifier):
            self._selection = self.scene().selectedItems()
        super().mousePressEvent(event)
    def mouseMoveEvent(self, event):
        super().mouseMoveEvent(event)
        if not self._clicked_node:
            for node in self._selection: node.setSelected(True)
    def mouseReleaseEvent(self, event):
        super().mouseReleaseEvent(event)
        for node in self._selection: node.setSelected(True)
        self.selectionChanged.emit()

    def setHighlighted(self, nodes):
        for node in nodes:
            node.setHighlighted(True)
        self.selectionChanged.emit()

    def getSelected(self):
        return self.scene().selectedItems()

    def getHighlighted(self):
        return [node for node in self.nodes
                if node.isHighlighted() and not node.isSelected()]

    def set_graph(self, graph):
        assert isinstance(graph, nx.Graph)
        self.setViewport(QtOpenGL.QGLWidget()
                         if len(graph) + graph.size() > 5000 and HAVE_OPENGL else
                         QtGui.QWidget())
        # TODO

        self.selectionChanged.emit()

    def addNode(self, node):
        assert isinstance(node, Node)
        self.nodes.append(node)
        self.scene().addItem(node)
    def addEdge(self, edge):
        assert isinstance(edge, Edge)
        self.edges.append(edge)
        self.scene().addItem(edge)

    def wheelEvent(self, event):
        self.scaleView(2**(event.delta() / 240))
    def scaleView(self, factor):
        magnitude = self.matrix().scale(factor, factor).mapRect(QRectF(0, 0, 1, 1)).width()
        if 0.2 < magnitude < 30:
            self.scale(factor, factor)
        # Reposition nodes' labels ...
        for node in self.nodes: node.adjust()
        # Reposition edges which are node-dependend (and nodes just "moved")
        for edge in self.edges: edge.adjust()

    def relayout(self):
        self._is_animating = True
        self.setCursor(Qt.ForbiddenCursor)
        ...

    def update_positions(self, positions):
        self.positionsChanged.emit(positions)
        return self._is_animating
    def _update_positions(self, positions):
        for node, pos in zip(self.nodes, positions):
            node.setPos(pos[0]*400, pos[1]*400)
            #~ node.setPos(pos[0], pos[1])
        qApp.processEvents()

# TODO: integrate with pyqtgraph

if __name__ == '__main__':

    import sys
    app = QtGui.QApplication(sys.argv)
    widget = GraphView()
    widget.show()
    import networkx as nx
    G = nx.scale_free_graph(int(sys.argv[1]) if len(sys.argv) > 1 else 100, seed=0)
    for v in sorted(G.nodes()):
        widget.addNode(Node(v, view=widget))
    for u, v in G.edges():
        widget.addEdge(Edge(widget.nodes[u], widget.nodes[v], view=widget))

    print('nodes', len(widget.nodes), 'edges', len(widget.edges))

    widget.relayout()
    #~ from threading import Thread
    #~ Thread(target=fruchterman_reingold_layout, args=(G,), kwargs=dict(iterations=50, callback=widget.update_positions)).start()
#~
    QtCore.QTimer.singleShot(0, lambda:
        fruchterman_reingold_layout(G,
                                    iterations=50,
                                    callback=widget.update_positions) and
        (lambda: widget.mousePressEvent(1))())
    QtCore.QTimer.singleShot(1000, lambda:
        [node.setHighlighted(True) for node in widget.nodes[:10]])

    sys.exit(app.exec_())



import Orange.network

from matplotlib import pyplot as plt

# create graph object of type GraphAsList
net = Orange.network.Network(5, 0)

# set edges
for i in range(4):
    for j in range(i + 1, 5):
        net[i, j] = 1

# vertices are placed randomly in NetworkOptimization constructor
networkOptimization = Orange.network.NetworkOptimization(net)


# read all edges and plot a line
for u, v in net.get_edges():
    x1, y1 = net.coors[0][u], net.coors[1][u]
    x2, y2 = net.coors[0][v], net.coors[1][v]
    plt.plot([x1, x2], [y1, y2], 'b-')

# read x and y coordinates to Python list
x = net.coors[0]
y = net.coors[1]

# plot vertices
plt.plot(x, y, 'ro')
plt.savefig("network-optimization.png")
